# Copyright (c) 2020, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=invalid-name, too-few-public-methods

"""Test cases for the zip module"""

from __future__ import unicode_literals
from __future__ import print_function

import logging # pylint: disable=unused-import
import unittest

from hpccm.templates.zipfile import zipfile

class Test_zipfile(unittest.TestCase):
    def setUp(self):
        """Disable logging output messages"""
        logging.disable(logging.ERROR)

    def test_missing_zipfile(self):
        """Missing zipfile option"""
        z = zipfile()
        self.assertEqual(z.unzip_step(), '')

    def test_filetypes(self):
        z = zipfile()
        self.assertEqual(z.unzip_step('foo.zip'), 'unzip foo.zip')

    def test_directory(self):
        """Directory specified"""
        z = zipfile()
        self.assertEqual(z.unzip_step('foo.zip', 'bar'),
                         'mkdir -p bar && unzip -d bar foo.zip')



# Copyright (c) 2018, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=invalid-name, too-few-public-methods

"""Test cases for the sed module"""

from __future__ import unicode_literals
from __future__ import print_function

import logging # pylint: disable=unused-import
import unittest

from hpccm.templates.sed import sed

class Test_sed(unittest.TestCase):
    def setUp(self):
        """Disable logging output messages"""
        logging.disable(logging.ERROR)

    def test_basic(self):
        """Basic sed"""
        s = sed()
        self.assertEqual(s.sed_step(file='foo',
                                    patterns=[r's/a/A/g',
                                              r's/FOO = BAR/FOO = BAZ/g']),
r'''sed -i -e s/a/A/g \
        -e 's/FOO = BAR/FOO = BAZ/g' foo''')

    def test_nofile(self):
        """No file specified"""
        s = sed()
        self.assertEqual(s.sed_step(patterns=[r's/a/A/g']), '')

    def test_nopatterns(self):
        """No patterns specified"""
        s = sed()
        self.assertEqual(s.sed_step(file='foo'), '')



# Copyright (c) 2020, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes

"""NCCL building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

from distutils.version import StrictVersion
import posixpath

import hpccm.templates.downloader
import hpccm.templates.envvars
import hpccm.templates.ldconfig

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.generic_build import generic_build
from hpccm.building_blocks.packages import packages
from hpccm.common import linux_distro
from hpccm.config import get_cpu_architecture
from hpccm.primitives.comment import comment
from hpccm.primitives.copy import copy
from hpccm.primitives.environment import environment

class nccl(bb_base, hpccm.templates.downloader, hpccm.templates.envvars,
           hpccm.templates.ldconfig):
    """The `nccl` building block installs the
    [NCCL](https://developer.nvidia.com/nccl) component.

    # Parameters

    branch: The git branch to clone.  Only recognized if the
    `repository` parameter is specified.  The default is empty, i.e.,
    use the default branch for the repository.

    build: Boolean flag to specify whether NCCL should be built from
    source.  The default value is False.

    commit: The git commit to clone.  Only recognized if the
    `repository` parameter is specified.  The default is empty, i.e.,
    use the latest commit on the default branch for the repository.

    cuda: Flag to specify the CUDA version of the package to download.
    The default is `11.6`.  This option is ignored if build is True.

    environment: Boolean flag to specify whether the environment
    (`CPATH`, `LD_LIBRARY_PATH`, `LIBRARY_PATH`, and `PATH`) should be
    modified to include NCCL. The default is True.  This option is
    ignored if build is False.

    make_variables: Dictionary of environment variables and values to
    set when building NCCL.  The default is an empty dictionary.  This
    option is ignored if build is False.

    ospackages: List of OS packages to install prior to building.  The
    default values are `make` and `wget`.

    prefix: The top level install location.  The default value is
    `/usr/local/nccl`.  This option is ignored if build is False.

    repository: The location of the git repository that should be used to build NCCL.  If True, then use the default `https://github.com/NVIDIA/nccl.git`
    repository.  The default is empty, i.e., use the release package
    specified by `version`.

    version: The version of NCCL to install.  The default value is
    `2.12.10-1`.

    # Examples

    ```python
    nccl(cuda='11.0', version='2.7.6-1')
    ```

    ```python
    nccl(build=True, version='2.7.6-1')
    ```

    """

    def __init__(self, **kwargs):
        """Initialize building block"""

        super(nccl, self).__init__(**kwargs)

        self.__baseurl = kwargs.pop('baseurl', 'https://github.com/NVIDIA/nccl/archive')
        self.__build = kwargs.pop('build', False)
        self.__build_environment = '' # Filled in by __configure
        self.__default_repository = 'https://github.com/NVIDIA/nccl.git'
        self.__distro_label = ''     # Filled in by __distro
        self.__cuda = kwargs.pop('cuda', '11.6')
        self.__make_variables = kwargs.pop('make_variables', {})
        self.__ospackages = kwargs.pop('ospackages', [])
        self.__prefix = kwargs.pop('prefix', '/usr/local/nccl')
        self.__src_directory = kwargs.pop('src_directory', None)
        self.__version = kwargs.pop('version', '2.12.10-1')
        self.__wd = kwargs.get('wd', hpccm.config.g_wd) # working directory

        if not self.__build:
            # Install prebuild package

            # Set the Linux distribution specific parameters
            self.__distro()

            self += comment('NCCL {}'.format(self.__version))
            self += packages(ospackages=self.__ospackages)
            self += packages(
                apt=['libnccl2={0}+cuda{1}'.format(self.__version,
                                                   self.__cuda),
                     'libnccl-dev={0}+cuda{1}'.format(self.__version,
                                                      self.__cuda)],
                apt_keys=['https://developer.download.nvidia.com/compute/cuda/repos/{0}/{1}/3bf863cc.pub'.format(self.__distro_label, get_cpu_architecture())],
                apt_repositories=['deb https://developer.download.nvidia.com/compute/cuda/repos/{0}/{1} /'.format(self.__distro_label, get_cpu_architecture())],
                yum=['libnccl-{0}+cuda{1}'.format(self.__version, self.__cuda),
                     'libnccl-devel-{0}+cuda{1}'.format(self.__version,
                                                        self.__cuda)],
                yum_keys=['https://developer.download.nvidia.com/compute/cuda/repos/{0}/{1}/3bf863cc.pub'.format(self.__distro_label, get_cpu_architecture())],
                yum_repositories=['https://developer.download.nvidia.com/compute/cuda/repos/{0}/{1}'.format(self.__distro_label, get_cpu_architecture())])

        else:
            # Build from source

            # Set the build options
            self.__configure()

            self.__download()
            kwargs['repository'] = self.repository
            kwargs['url'] = self.url

            # Setup the environment variables
            self.environment_variables['CPATH'] = '{}:$CPATH'.format(
                posixpath.join(self.__prefix, 'include'))
            self.environment_variables['LIBRARY_PATH'] = '{}:$LIBRARY_PATH'.format(
                posixpath.join(self.__prefix, 'lib'))
            self.environment_variables['PATH'] = '{}:$PATH'.format(
                posixpath.join(self.__prefix, 'bin'))
            if not self.ldconfig:
                self.environment_variables['LD_LIBRARY_PATH'] = '{}:$LD_LIBRARY_PATH'.format(posixpath.join(self.__prefix, 'lib'))

            self.__bb = generic_build(
                base_annotation=self.__class__.__name__,
                build = ['{} make -j$(nproc) install'.format(
                    self.__build_environment)],
                comment=False,
                devel_environment=self.environment_variables,
                directory='nccl-{}'.format(self.__version) if not self.repository else None,
                prefix=self.__prefix,
                runtime_environment=self.environment_variables,
                **kwargs)

            self += comment('NCCL')
            self += packages(ospackages=self.__ospackages)
            self += self.__bb

    def __configure(self):
        """Setup build options based on user parameters"""

        e = {}

        e['PREFIX'] = self.__prefix

        if self.__make_variables:
          e.update(self.__make_variables)

        l = []
        if e:
            for key, val in sorted(e.items()):
                l.append('{0}={1}'.format(key, val))

        self.__build_environment = ' '.join(l)

    def __distro(self):
        """Based on the Linux distribution, set values accordingly.  A user
        specified value overrides any defaults."""

        if hpccm.config.g_linux_distro == linux_distro.UBUNTU:
            if not self.__ospackages:
                self.__ospackages = ['apt-transport-https', 'ca-certificates',
                                     'gnupg', 'wget']

            if hpccm.config.g_linux_version >= StrictVersion('18.0'):
                self.__distro_label = 'ubuntu1804'
            else:
                self.__distro_label = 'ubuntu1604'

        elif hpccm.config.g_linux_distro == linux_distro.CENTOS:
            if hpccm.config.g_linux_version >= StrictVersion('8.0'):
                self.__distro_label = 'rhel8'
            else:
                self.__distro_label = 'rhel7'

        else: # pragma: no cover
            raise RuntimeError('Unknown Linux distribution')

    def __download(self):
        """Set download source based on user parameters"""

        if not self.__ospackages:
            self.__ospackages = ['make', 'wget']

            if hpccm.config.g_linux_distro == linux_distro.CENTOS:
                self.__ospackages.append('which')

            if self.repository:
                self.__ospackages.append('git')

        # Use the default repository if set to True
        if self.repository is True:
            self.repository = self.__default_repository

        if not self.repository and not self.url:
            self.url = '{0}/v{1}.tar.gz'.format(self.__baseurl, self.__version)

    def runtime(self, _from='0'):
        """Generate the set of instructions to install the runtime specific
        components from a build in a previous stage.

        # Examples

        ```python
        n = nccl(...)
        Stage0 += n
        Stage1 += n.runtime()
        ```
        """
        self.rt += comment('NCCL')
        if self.__build:
            self.rt += copy(_from=_from, src=self.__prefix, dest=self.__prefix)
            self.rt += environment(variables=self.environment_step())
        else:
            self.rt += packages(ospackages=self.__ospackages)
            self.rt += packages(
                apt=['libnccl2={0}+cuda{1}'.format(self.__version,
                                                   self.__cuda)],
                apt_keys=['https://developer.download.nvidia.com/compute/cuda/repos/{0}/{1}/3bf863cc.pub'.format(self.__distro_label, get_cpu_architecture())],
                apt_repositories=['deb https://developer.download.nvidia.com/compute/cuda/repos/{0}/{1} /'.format(self.__distro_label, get_cpu_architecture())],
                yum=['libnccl-{0}+cuda{1}'.format(self.__version, self.__cuda)],
                yum_keys=['https://developer.download.nvidia.com/compute/cuda/repos/{0}/{1}/3bf863cc.pub'.format(self.__distro_label, get_cpu_architecture())],
                yum_repositories=['https://developer.download.nvidia.com/compute/cuda/repos/{0}/{1}'.format(self.__distro_label, get_cpu_architecture())])

        return str(self.rt)



# Copyright (c) 2020, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes

"""MAGMA building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import posixpath

import hpccm.templates.envvars
import hpccm.templates.ldconfig

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.generic_cmake import generic_cmake
from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment

class magma(bb_base, hpccm.templates.envvars, hpccm.templates.ldconfig):
    """The `magma` building block configures, builds, and installs the
    [MAGMA](https://icl.cs.utk.edu/magma) component.

    The [CMake](#cmake) building block should be installed prior to
    this building block.

    Either the [MKL](#mkl) or [OpenBLAS](#openblas) building block
    should also be installed.

    # Parameters

    annotate: Boolean flag to specify whether to include annotations
    (labels).  The default is False.

    cmake_opts: List of options to pass to `cmake`.  The default value
    is an empty list.

    gpu_target: List of GPU architectures to compile.  The default
    values are `Pascal`, `Volta`, and `Turing`.

    ospackages: List of OS packages to install prior to configuring
    and building.  The default values are `tar` and `wget`.

    prefix: The top level install location.  The default value is
    `/usr/local/magma`.

    toolchain: The toolchain object.  This should be used if
    non-default compilers or other toolchain options are needed.  The
    default is empty.

    version: The version of MAGMA source to download.  The default
    value is `2.5.3`.

    # Examples

    ```python
    magma(prefix='/opt/magma', version='2.5.3')
    ```

    """

    def __init__(self, **kwargs):
        """Initialize building block"""

        super(magma, self).__init__(**kwargs)

        self.__baseurl = kwargs.pop('baseurl', 'http://icl.utk.edu/projectsfiles/magma/downloads')
        self.__cmake_opts = kwargs.pop('cmake_opts', [])
        self.__gpu_target = kwargs.pop('gpu_target',
                                       ['Pascal', 'Volta', 'Turing'])
        self.__ospackages = kwargs.pop('ospackages', ['tar', 'wget'])
        self.__prefix = kwargs.pop('prefix', '/usr/local/magma')
        self.__version = kwargs.pop('version', '2.5.3')

        # Set the cmake options
        self.__cmake()

        # Setup the environment variables
        self.environment_variables['CPATH'] = '{}:$CPATH'.format(
            posixpath.join(self.__prefix, 'include'))
        self.environment_variables['LIBRARY_PATH'] = '{}:$LIBRARY_PATH'.format(
            posixpath.join(self.__prefix, 'lib'))
        if not self.ldconfig:
            self.environment_variables['LD_LIBRARY_PATH'] = '{}:$LD_LIBRARY_PATH'.format(posixpath.join(self.__prefix, 'lib'))

        # Setup build configuration
        self.__bb = generic_cmake(
            annotations={'version': self.__version},
            base_annotation=self.__class__.__name__,
            comment=False,
            cmake_opts=self.__cmake_opts,
            devel_environment=self.environment_variables,
            prefix=self.__prefix,
            runtime_environment=self.environment_variables,
            url='{0}/magma-{1}.tar.gz'.format(self.__baseurl, self.__version),
            **kwargs)

        # Container instructions
        self += comment('MAGMA version {}'.format(self.__version))
        self += packages(ospackages=self.__ospackages)
        self += self.__bb

    def __cmake(self):
        """Setup cmake options based on users parameters"""

        # GPU architectures
        if self.__gpu_target:
            self.__cmake_opts.append('-DGPU_TARGET="{}"'.format(
                ' '.join(self.__gpu_target)))

    def runtime(self, _from='0'):
        """Generate the set of instructions to install the runtime specific
        components from a build in a previous stage.

        # Examples
        ```python
        m = magma(...)
        Stage0 += m
        Stage1 += m.runtime()
        ```
        """
        self.rt += comment('MAGMA')
        self.rt += self.__bb.runtime(_from=_from)
        return str(self.rt)



# Copyright (c) 2019, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=invalid-name, too-few-public-methods, bad-continuation

"""Test cases for the catalyst module"""

from __future__ import unicode_literals
from __future__ import print_function

import logging # pylint: disable=unused-import
import unittest

from helpers import centos, docker, ubuntu

from hpccm.building_blocks.catalyst import catalyst
from hpccm.toolchain import toolchain

class Test_catalyst(unittest.TestCase):
    def setUp(self):
        """Disable logging output messages"""
        logging.disable(logging.ERROR)

    @ubuntu
    @docker
    def test_defaults_ubuntu(self):
        """Default catalyst building block"""
        c = catalyst()
        self.assertEqual(str(c),
r'''# ParaView Catalyst version 5.6.1
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        git \
        gzip \
        libgl1-mesa-dev \
        libice-dev \
        libsm-dev \
        libx11-dev \
        libxau-dev \
        libxext-dev \
        libxt-dev \
        make \
        tar \
        wget && \
    rm -rf /var/lib/apt/lists/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -O /var/tmp/Catalyst-v5.6.1-Base-Enable-Python-Essentials-Extras-Rendering-Base.tar.gz -P /var/tmp https://www.paraview.org/paraview-downloads/download.php?submit=Download\&version=v5.6\&type=catalyst\&os=Sources\&downloadFile=Catalyst-v5.6.1-Base-Enable-Python-Essentials-Extras-Rendering-Base.tar.gz && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/Catalyst-v5.6.1-Base-Enable-Python-Essentials-Extras-Rendering-Base.tar.gz -C /var/tmp -z && \
    mkdir -p /var/tmp/Catalyst-v5.6.1-Base-Enable-Python-Essentials-Extras-Rendering-Base/build && cd /var/tmp/Catalyst-v5.6.1-Base-Enable-Python-Essentials-Extras-Rendering-Base/build && /var/tmp/Catalyst-v5.6.1-Base-Enable-Python-Essentials-Extras-Rendering-Base/cmake.sh -DCMAKE_INSTALL_PREFIX=/usr/local/catalyst /var/tmp/Catalyst-v5.6.1-Base-Enable-Python-Essentials-Extras-Rendering-Base && \
    cmake --build /var/tmp/Catalyst-v5.6.1-Base-Enable-Python-Essentials-Extras-Rendering-Base/build --target all -- -j$(nproc) && \
    cmake --build /var/tmp/Catalyst-v5.6.1-Base-Enable-Python-Essentials-Extras-Rendering-Base/build --target install -- -j$(nproc) && \
    rm -rf /var/tmp/Catalyst-v5.6.1-Base-Enable-Python-Essentials-Extras-Rendering-Base.tar.gz /var/tmp/Catalyst-v5.6.1-Base-Enable-Python-Essentials-Extras-Rendering-Base
ENV LD_LIBRARY_PATH=/usr/local/catalyst/lib:$LD_LIBRARY_PATH \
    PATH=/usr/local/catalyst/bin:$PATH''')

    @centos
    @docker
    def test_defaults_centos(self):
        """Default catalyst building block"""
        c = catalyst()
        self.assertEqual(str(c),
r'''# ParaView Catalyst version 5.6.1
RUN yum install -y \
        git \
        gzip \
        libICE-devel \
        libSM-devel \
        libX11-devel \
        libXau-devel \
        libXext-devel \
        libXt-devel \
        libglvnd-devel \
        make \
        mesa-libGL-devel \
        tar \
        wget \
        which && \
    rm -rf /var/cache/yum/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -O /var/tmp/Catalyst-v5.6.1-Base-Enable-Python-Essentials-Extras-Rendering-Base.tar.gz -P /var/tmp https://www.paraview.org/paraview-downloads/download.php?submit=Download\&version=v5.6\&type=catalyst\&os=Sources\&downloadFile=Catalyst-v5.6.1-Base-Enable-Python-Essentials-Extras-Rendering-Base.tar.gz && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/Catalyst-v5.6.1-Base-Enable-Python-Essentials-Extras-Rendering-Base.tar.gz -C /var/tmp -z && \
    mkdir -p /var/tmp/Catalyst-v5.6.1-Base-Enable-Python-Essentials-Extras-Rendering-Base/build && cd /var/tmp/Catalyst-v5.6.1-Base-Enable-Python-Essentials-Extras-Rendering-Base/build && /var/tmp/Catalyst-v5.6.1-Base-Enable-Python-Essentials-Extras-Rendering-Base/cmake.sh -DCMAKE_INSTALL_PREFIX=/usr/local/catalyst /var/tmp/Catalyst-v5.6.1-Base-Enable-Python-Essentials-Extras-Rendering-Base && \
    cmake --build /var/tmp/Catalyst-v5.6.1-Base-Enable-Python-Essentials-Extras-Rendering-Base/build --target all -- -j$(nproc) && \
    cmake --build /var/tmp/Catalyst-v5.6.1-Base-Enable-Python-Essentials-Extras-Rendering-Base/build --target install -- -j$(nproc) && \
    rm -rf /var/tmp/Catalyst-v5.6.1-Base-Enable-Python-Essentials-Extras-Rendering-Base.tar.gz /var/tmp/Catalyst-v5.6.1-Base-Enable-Python-Essentials-Extras-Rendering-Base
ENV LD_LIBRARY_PATH=/usr/local/catalyst/lib:$LD_LIBRARY_PATH \
    PATH=/usr/local/catalyst/bin:$PATH''')

    @ubuntu
    @docker
    def test_edition(self):
        """edition option"""
        c = catalyst(edition='Base-Essentials')
        self.assertEqual(str(c),
r'''# ParaView Catalyst version 5.6.1
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        git \
        gzip \
        make \
        tar \
        wget && \
    rm -rf /var/lib/apt/lists/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -O /var/tmp/Catalyst-v5.6.1-Base-Essentials.tar.gz -P /var/tmp https://www.paraview.org/paraview-downloads/download.php?submit=Download\&version=v5.6\&type=catalyst\&os=Sources\&downloadFile=Catalyst-v5.6.1-Base-Essentials.tar.gz && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/Catalyst-v5.6.1-Base-Essentials.tar.gz -C /var/tmp -z && \
    mkdir -p /var/tmp/Catalyst-v5.6.1-Base-Essentials/build && cd /var/tmp/Catalyst-v5.6.1-Base-Essentials/build && /var/tmp/Catalyst-v5.6.1-Base-Essentials/cmake.sh -DCMAKE_INSTALL_PREFIX=/usr/local/catalyst /var/tmp/Catalyst-v5.6.1-Base-Essentials && \
    cmake --build /var/tmp/Catalyst-v5.6.1-Base-Essentials/build --target all -- -j$(nproc) && \
    cmake --build /var/tmp/Catalyst-v5.6.1-Base-Essentials/build --target install -- -j$(nproc) && \
    rm -rf /var/tmp/Catalyst-v5.6.1-Base-Essentials.tar.gz /var/tmp/Catalyst-v5.6.1-Base-Essentials
ENV LD_LIBRARY_PATH=/usr/local/catalyst/lib:$LD_LIBRARY_PATH \
    PATH=/usr/local/catalyst/bin:$PATH''')

    @ubuntu
    @docker
    def test_ldconfig(self):
        """ldconfig option"""
        c = catalyst(ldconfig=True, version='5.6.0')
        self.assertEqual(str(c),
r'''# ParaView Catalyst version 5.6.0
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        git \
        gzip \
        libgl1-mesa-dev \
        libice-dev \
        libsm-dev \
        libx11-dev \
        libxau-dev \
        libxext-dev \
        libxt-dev \
        make \
        tar \
        wget && \
    rm -rf /var/lib/apt/lists/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -O /var/tmp/Catalyst-v5.6.0-Base-Enable-Python-Essentials-Extras-Rendering-Base.tar.gz -P /var/tmp https://www.paraview.org/paraview-downloads/download.php?submit=Download\&version=v5.6\&type=catalyst\&os=Sources\&downloadFile=Catalyst-v5.6.0-Base-Enable-Python-Essentials-Extras-Rendering-Base.tar.gz && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/Catalyst-v5.6.0-Base-Enable-Python-Essentials-Extras-Rendering-Base.tar.gz -C /var/tmp -z && \
    mkdir -p /var/tmp/Catalyst-v5.6.0-Base-Enable-Python-Essentials-Extras-Rendering-Base/build && cd /var/tmp/Catalyst-v5.6.0-Base-Enable-Python-Essentials-Extras-Rendering-Base/build && /var/tmp/Catalyst-v5.6.0-Base-Enable-Python-Essentials-Extras-Rendering-Base/cmake.sh -DCMAKE_INSTALL_PREFIX=/usr/local/catalyst /var/tmp/Catalyst-v5.6.0-Base-Enable-Python-Essentials-Extras-Rendering-Base && \
    cmake --build /var/tmp/Catalyst-v5.6.0-Base-Enable-Python-Essentials-Extras-Rendering-Base/build --target all -- -j$(nproc) && \
    cmake --build /var/tmp/Catalyst-v5.6.0-Base-Enable-Python-Essentials-Extras-Rendering-Base/build --target install -- -j$(nproc) && \
    echo "/usr/local/catalyst/lib" >> /etc/ld.so.conf.d/hpccm.conf && ldconfig && \
    rm -rf /var/tmp/Catalyst-v5.6.0-Base-Enable-Python-Essentials-Extras-Rendering-Base.tar.gz /var/tmp/Catalyst-v5.6.0-Base-Enable-Python-Essentials-Extras-Rendering-Base
ENV PATH=/usr/local/catalyst/bin:$PATH''')

    @ubuntu
    @docker
    def test_runtime(self):
        """Runtime"""
        c = catalyst()
        r = c.runtime()
        self.assertEqual(r,
r'''# ParaView Catalyst
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        libgl1-mesa-glx \
        libice6 \
        libsm6 \
        libx11-6 \
        libxau6 \
        libxext6 \
        libxt6 && \
    rm -rf /var/lib/apt/lists/*
COPY --from=0 /usr/local/catalyst /usr/local/catalyst
ENV LD_LIBRARY_PATH=/usr/local/catalyst/lib:$LD_LIBRARY_PATH \
    PATH=/usr/local/catalyst/bin:$PATH''')



# Copyright (c) 2018, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes

"""KNEM building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import posixpath

import hpccm.templates.envvars

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.generic_build import generic_build
from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment

class knem(bb_base, hpccm.templates.envvars):
    """The `knem` building block install the headers from the
    [KNEM](http://knem.gforge.inria.fr) component.

    # Parameters

    annotate: Boolean flag to specify whether to include annotations
    (labels).  The default is False.

    environment: Boolean flag to specify whether the environment
    (`CPATH`) should be modified to include knem. The default is True.

    ospackages: List of OS packages to install prior to installing.
    The default values are `ca-certificates` and `git`.

    prefix: The top level install location.  The default value is
    `/usr/local/knem`.

    version: The version of KNEM source to download.  The default
    value is `1.1.4`.

    # Examples

    ```python
    knem(prefix='/opt/knem/1.1.3', version='1.1.3')
    ```

    """

    def __init__(self, **kwargs):
        """Initialize building block"""

        super(knem, self).__init__(**kwargs)

        self.__ospackages = kwargs.pop('ospackages',
                                       ['ca-certificates', 'git'])
        self.__prefix = kwargs.pop('prefix', '/usr/local/knem')
        self.__repository = kwargs.pop('repository',
                                       'https://gitlab.inria.fr/knem/knem.git')
        self.__version = kwargs.pop('version', '1.1.4')

        # Setup the environment variables
        self.environment_variables['CPATH'] = '{}:$CPATH'.format(
            posixpath.join(self.__prefix, 'include'))

        # Setup build configuration
        self.__bb = generic_build(
            base_annotation=self.__class__.__name__,
            branch='knem-{}'.format(self.__version),
            comment=False,
            devel_environment=self.environment_variables,
            install=['mkdir -p {}/include'.format(self.__prefix),
                     'cp common/*.h {}/include'.format(self.__prefix)],
            runtime_environment=self.environment_variables,
            prefix=self.__prefix,
            repository=self.__repository,
            **kwargs)

        # Container instructions
        self += comment('KNEM version {}'.format(self.__version))
        self += packages(ospackages=self.__ospackages)
        self += self.__bb

    def runtime(self, _from='0'):
        """Generate the set of instructions to install the runtime specific
        components from a build in a previous stage.

        # Examples

        ```python
        k = knem(...)
        Stage0 += k
        Stage1 += k.runtime()
        ```
        """
        self.rt += comment('KNEM')
        self.rt += self.__bb.runtime(_from=_from)
        return str(self.rt)



# Copyright (c) 2020, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=invalid-name, too-few-public-methods

"""Test cases for the downloader module"""

from __future__ import unicode_literals
from __future__ import print_function

import logging # pylint: disable=unused-import
import unittest

from helpers import docker, singularity, bash

from hpccm.templates.downloader import downloader

class Test_downloader(unittest.TestCase):
    def setUp(self):
        """Disable logging output messages"""
        logging.disable(logging.ERROR)

    def test_missing_url(self):
        """Missing url option"""
        d = downloader()
        with self.assertRaises(RuntimeError):
            d.download_step()

    def test_url_and_repository(self):
        """Both URL and repository specified"""
        d = downloader(repository='https://github.com/foo/bar',
                       url='http://mysite.com/foo.tgz')
        with self.assertRaises(RuntimeError):
            d.download_step()

    @docker
    def test_basic_wget_docker(self):
        """Basic wget"""
        d = downloader(url='http://mysite.com/foo.tgz')
        self.assertEqual(d.download_step(),
r'''mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp http://mysite.com/foo.tgz && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/foo.tgz -C /var/tmp -z''')
        self.assertEqual(d.src_directory, '/var/tmp/foo')

    @singularity
    def test_basic_wget_singularity(self):
        """Basic wget"""
        d = downloader(url='http://mysite.com/foo.tgz')
        self.assertEqual(d.download_step(),
r'''mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp http://mysite.com/foo.tgz
    mkdir -p /var/tmp && tar -x -f /var/tmp/foo.tgz -C /var/tmp -z''')
        self.assertEqual(d.src_directory, '/var/tmp/foo')

    @bash
    def test_basic_wget_bash(self):
        """Basic wget"""
        d = downloader(url='http://mysite.com/foo.tgz')
        self.assertEqual(d.download_step(),
r'''mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp http://mysite.com/foo.tgz
mkdir -p /var/tmp && tar -x -f /var/tmp/foo.tgz -C /var/tmp -z''')
        self.assertEqual(d.src_directory, '/var/tmp/foo')

    @docker
    def test_bad_url(self):
        """Unrecognized package format, assumes tar can figure it out"""
        d = downloader(url='http://mysite.com/foo.Z')
        self.assertEqual(d.download_step(),
r'''mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp http://mysite.com/foo.Z && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/foo.Z -C /var/tmp''')
        self.assertEqual(d.src_directory, None)

    @docker
    def test_bad_url_noallow(self):
        """Unrecognzied package format"""
        d = downloader(url='http://mysite.com/foo.Z')
        with self.assertRaises(RuntimeError):
            d.download_step(allow_unknown_filetype=False)

    @docker
    def test_basic_git(self):
        """Basic git"""
        d = downloader(repository='https://github.com/foo/bar.git')
        self.assertEqual(d.download_step(),
r'''mkdir -p /var/tmp && cd /var/tmp && git clone --depth=1 https://github.com/foo/bar.git bar && cd -''')
        self.assertEqual(d.src_directory, '/var/tmp/bar')

    @docker
    def test_git_recursive_branch(self):
        """Recursive clone of a branch"""
        d = downloader(branch='dev', repository='https://github.com/foo/bar')
        self.assertEqual(d.download_step(recursive=True),
r'''mkdir -p /var/tmp && cd /var/tmp && git clone --depth=1 --branch dev --recursive https://github.com/foo/bar bar && cd -''')

    @docker
    def test_git_commit_wd(self):
        """Git commit into an alternative working directory"""
        d = downloader(commit='deadbeef',
                       repository='https://github.com/foo/bar')
        self.assertEqual(d.download_step(wd='/tmp/git'),
r'''mkdir -p /tmp/git && cd /tmp/git && git clone  https://github.com/foo/bar bar && cd - && cd /tmp/git/bar && git checkout deadbeef && cd -''')
        self.assertEqual(d.src_directory, '/tmp/git/bar')

    @docker
    def test_tarball(self):
        """Local tarball, no download"""
        d = downloader(package='foo.tar.gz')
        self.assertEqual(d.download_step(), 'mkdir -p /var/tmp && tar -x -f /var/tmp/foo.tar.gz -C /var/tmp -z')
        self.assertEqual(d.src_directory, '/var/tmp/foo')

    @docker
    def test_zipfile(self):
        """Local zipfile, no download"""
        d = downloader(package='foo.zip')
        self.assertEqual(d.download_step(), 'mkdir -p /var/tmp && unzip -d /var/tmp /var/tmp/foo.zip')
        self.assertEqual(d.src_directory, '/var/tmp/foo')



# Copyright (c) 2020, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=invalid-name, too-few-public-methods, bad-continuation

"""Test cases for the nvshmem module"""

from __future__ import unicode_literals
from __future__ import print_function

import logging # pylint: disable=unused-import
import unittest

from helpers import centos, docker, ubuntu

from hpccm.building_blocks.nvshmem import nvshmem

class Test_nvshmem(unittest.TestCase):
    def setUp(self):
        """Disable logging output messages"""
        logging.disable(logging.ERROR)

    @ubuntu
    @docker
    def test_defaults_ubuntu(self):
        """nvshmem defaults"""
        n = nvshmem()
        self.assertEqual(str(n),
r'''# NVSHMEM 2.2.1
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        make \
        wget && \
    rm -rf /var/lib/apt/lists/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://developer.download.nvidia.com/compute/redist/nvshmem/2.2.1/source/nvshmem_src_2.2.1-0.txz && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/nvshmem_src_2.2.1-0.txz -C /var/tmp -J && \
    cd /var/tmp/nvshmem_src_2.2.1-0 && \
    CUDA_HOME=/usr/local/cuda NVSHMEM_MPI_SUPPORT=0 NVSHMEM_PREFIX=/usr/local/nvshmem make -j$(nproc) install && \
    rm -rf /var/tmp/nvshmem_src_2.2.1-0 /var/tmp/nvshmem_src_2.2.1-0.txz
ENV CPATH=/usr/local/nvshmem/include:$CPATH \
    LD_LIBRARY_PATH=/usr/local/nvshmem/lib:$LD_LIBRARY_PATH \
    LIBRARY_PATH=/usr/local/nvshmem/lib:$LIBRARY_PATH \
    PATH=/usr/local/nvshmem/bin:$PATH''')

    @ubuntu
    @docker
    def test_binary_tarball_ubuntu(self):
        """nvshmem binary tarball"""
        n = nvshmem(binary_tarball='nvshmem_0.4.1-0+cuda10_x86_64.txz')
        self.assertEqual(str(n),
r'''# NVSHMEM
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        make \
        wget && \
    rm -rf /var/lib/apt/lists/*
COPY nvshmem_0.4.1-0+cuda10_x86_64.txz /var/tmp/nvshmem_0.4.1-0+cuda10_x86_64.txz
RUN mkdir -p /usr/local/nvshmem && tar -x -f /var/tmp/nvshmem_0.4.1-0+cuda10_x86_64.txz -C /usr/local/nvshmem -J --strip-components=1 && \
    rm -rf /var/tmp/nvshmem_0.4.1-0+cuda10_x86_64.txz
ENV CPATH=/usr/local/nvshmem/include:$CPATH \
    LD_LIBRARY_PATH=/usr/local/nvshmem/lib:$LD_LIBRARY_PATH \
    LIBRARY_PATH=/usr/local/nvshmem/lib:$LIBRARY_PATH \
    PATH=/usr/local/nvshmem/bin:$PATH''')

    @centos
    @docker
    def test_binary_tarball_centos(self):
        """nvshmem binary tarball"""
        n = nvshmem(binary_tarball='nvshmem_0.4.1-0+cuda10_x86_64.txz')
        self.assertEqual(str(n),
r'''# NVSHMEM
RUN yum install -y \
        make \
        wget && \
    rm -rf /var/cache/yum/*
COPY nvshmem_0.4.1-0+cuda10_x86_64.txz /var/tmp/nvshmem_0.4.1-0+cuda10_x86_64.txz
RUN mkdir -p /usr/local/nvshmem && tar -x -f /var/tmp/nvshmem_0.4.1-0+cuda10_x86_64.txz -C /usr/local/nvshmem -J --strip-components=1 && \
    rm -rf /var/tmp/nvshmem_0.4.1-0+cuda10_x86_64.txz
ENV CPATH=/usr/local/nvshmem/include:$CPATH \
    LD_LIBRARY_PATH=/usr/local/nvshmem/lib:$LD_LIBRARY_PATH \
    LIBRARY_PATH=/usr/local/nvshmem/lib:$LIBRARY_PATH \
    PATH=/usr/local/nvshmem/bin:$PATH''')

    @ubuntu
    @docker
    def test_package_ubuntu(self):
        """nvshmem source package"""
        n = nvshmem(package='nvshmem_src_2.1.2-0.txz')
        self.assertEqual(str(n),
r'''# NVSHMEM
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        make \
        wget && \
    rm -rf /var/lib/apt/lists/*
COPY nvshmem_src_2.1.2-0.txz /var/tmp/nvshmem_src_2.1.2-0.txz
RUN mkdir -p /var/tmp && tar -x -f /var/tmp/nvshmem_src_2.1.2-0.txz -C /var/tmp -J && \
    cd /var/tmp/nvshmem_src_2.1.2-0 && \
    CUDA_HOME=/usr/local/cuda NVSHMEM_MPI_SUPPORT=0 NVSHMEM_PREFIX=/usr/local/nvshmem make -j$(nproc) install && \
    rm -rf /var/tmp/nvshmem_src_2.1.2-0 /var/tmp/nvshmem_src_2.1.2-0.txz
ENV CPATH=/usr/local/nvshmem/include:$CPATH \
    LD_LIBRARY_PATH=/usr/local/nvshmem/lib:$LD_LIBRARY_PATH \
    LIBRARY_PATH=/usr/local/nvshmem/lib:$LIBRARY_PATH \
    PATH=/usr/local/nvshmem/bin:$PATH''')

    @centos
    @docker
    def test_package_options_centos(self):
        """nvshmem source package with all options"""
        n = nvshmem(gdrcopy='/usr/local/gdrcopy', hydra=True,
                    make_variables={
                        'NVCC_GENCODE': '-gencode=arch=compute_70,code=sm_70',
                        'NVSHMEM_VERBOSE': 1},
                    mpi='/usr/local/openmpi',
                    package='nvshmem_src_2.1.2-0.txz',
                    shmem='/usr/local/openmpi')
        self.assertEqual(str(n),
r'''# NVSHMEM
RUN yum install -y \
        automake \
        make \
        wget && \
    rm -rf /var/cache/yum/*
COPY nvshmem_src_2.1.2-0.txz /var/tmp/nvshmem_src_2.1.2-0.txz
RUN mkdir -p /var/tmp && tar -x -f /var/tmp/nvshmem_src_2.1.2-0.txz -C /var/tmp -J && \
    cd /var/tmp/nvshmem_src_2.1.2-0 && \
    CUDA_HOME=/usr/local/cuda GDRCOPY_HOME=/usr/local/gdrcopy MPI_HOME=/usr/local/openmpi NVCC_GENCODE=-gencode=arch=compute_70,code=sm_70 NVSHMEM_MPI_SUPPORT=1 NVSHMEM_PREFIX=/usr/local/nvshmem NVSHMEM_SHMEM_SUPPORT=1 NVSHMEM_VERBOSE=1 SHMEM_HOME=/usr/local/openmpi make -j$(nproc) install && \
    ./scripts/install_hydra.sh /var/tmp /usr/local/nvshmem && \
    rm -rf /var/tmp/nvshmem_src_2.1.2-0 /var/tmp/nvshmem_src_2.1.2-0.txz
ENV CPATH=/usr/local/nvshmem/include:$CPATH \
    LD_LIBRARY_PATH=/usr/local/nvshmem/lib:$LD_LIBRARY_PATH \
    LIBRARY_PATH=/usr/local/nvshmem/lib:$LIBRARY_PATH \
    PATH=/usr/local/nvshmem/bin:$PATH''')

    @ubuntu
    @docker
    def test_binary_runtime(self):
        """Runtime"""
        n = nvshmem(binary_tarball='nvshmem_0.4.1-0+cuda10_x86_64.txz')
        r = n.runtime()
        self.assertEqual(r,
r'''# NVSHMEM
COPY --from=0 /usr/local/nvshmem /usr/local/nvshmem
ENV CPATH=/usr/local/nvshmem/include:$CPATH \
    LD_LIBRARY_PATH=/usr/local/nvshmem/lib:$LD_LIBRARY_PATH \
    LIBRARY_PATH=/usr/local/nvshmem/lib:$LIBRARY_PATH \
    PATH=/usr/local/nvshmem/bin:$PATH''')

    @ubuntu
    @docker
    def test_source_runtime(self):
        """Runtime"""
        n = nvshmem(package='nvshmem_src_2.1.2-0.txz')
        r = n.runtime()
        self.assertEqual(r,
r'''# NVSHMEM
COPY --from=0 /usr/local/nvshmem /usr/local/nvshmem
ENV CPATH=/usr/local/nvshmem/include:$CPATH \
    LD_LIBRARY_PATH=/usr/local/nvshmem/lib:$LD_LIBRARY_PATH \
    LIBRARY_PATH=/usr/local/nvshmem/lib:$LIBRARY_PATH \
    PATH=/usr/local/nvshmem/bin:$PATH''')



# Copyright (c) 2018, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=invalid-name, too-few-public-methods, bad-continuation

"""Test cases for the netcdf module"""

from __future__ import unicode_literals
from __future__ import print_function

import logging # pylint: disable=unused-import
import unittest

from helpers import centos, docker, ubuntu

from hpccm.building_blocks.netcdf import netcdf

class Test_netcdf(unittest.TestCase):
    def setUp(self):
        """Disable logging output messages"""
        logging.disable(logging.ERROR)

    @ubuntu
    @docker
    def test_defaults_ubuntu(self):
        """Default netcdf building block"""
        n = netcdf()
        self.assertEqual(str(n),
r'''# NetCDF version 4.7.4, NetCDF C++ version 4.3.1, NetCDF Fortran
# version 4.5.3
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        ca-certificates \
        file \
        libcurl4-openssl-dev \
        m4 \
        make \
        wget \
        zlib1g-dev && \
    rm -rf /var/lib/apt/lists/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://github.com/Unidata/netcdf-c/archive/v4.7.4.tar.gz && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/v4.7.4.tar.gz -C /var/tmp -z && \
    cd /var/tmp/netcdf-c-4.7.4 &&   ./configure --prefix=/usr/local/netcdf && \
    make -j$(nproc) && \
    make -j$(nproc) install && \
    rm -rf /var/tmp/netcdf-c-4.7.4 /var/tmp/v4.7.4.tar.gz
ENV CPATH=/usr/local/netcdf/include:$CPATH \
    LD_LIBRARY_PATH=/usr/local/netcdf/lib:$LD_LIBRARY_PATH \
    LIBRARY_PATH=/usr/local/netcdf/lib:$LIBRARY_PATH \
    PATH=/usr/local/netcdf/bin:$PATH
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://github.com/Unidata/netcdf-cxx4/archive/v4.3.1.tar.gz && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/v4.3.1.tar.gz -C /var/tmp -z && \
    cd /var/tmp/netcdf-cxx4-4.3.1 &&   ./configure --prefix=/usr/local/netcdf && \
    make -j$(nproc) && \
    make -j$(nproc) install && \
    rm -rf /var/tmp/netcdf-cxx4-4.3.1 /var/tmp/v4.3.1.tar.gz
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://github.com/Unidata/netcdf-fortran/archive/v4.5.3.tar.gz && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/v4.5.3.tar.gz -C /var/tmp -z && \
    cd /var/tmp/netcdf-fortran-4.5.3 &&   ./configure --prefix=/usr/local/netcdf && \
    make -j$(nproc) && \
    make -j$(nproc) install && \
    rm -rf /var/tmp/netcdf-fortran-4.5.3 /var/tmp/v4.5.3.tar.gz''')

    @centos
    @docker
    def test_defaults_centos(self):
        """Default netcdf building block"""
        n = netcdf()
        self.assertEqual(str(n),
r'''# NetCDF version 4.7.4, NetCDF C++ version 4.3.1, NetCDF Fortran
# version 4.5.3
RUN yum install -y \
        ca-certificates \
        file \
        libcurl-devel \
        m4 \
        make \
        wget \
        zlib-devel && \
    rm -rf /var/cache/yum/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://github.com/Unidata/netcdf-c/archive/v4.7.4.tar.gz && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/v4.7.4.tar.gz -C /var/tmp -z && \
    cd /var/tmp/netcdf-c-4.7.4 &&   ./configure --prefix=/usr/local/netcdf && \
    make -j$(nproc) && \
    make -j$(nproc) install && \
    rm -rf /var/tmp/netcdf-c-4.7.4 /var/tmp/v4.7.4.tar.gz
ENV CPATH=/usr/local/netcdf/include:$CPATH \
    LD_LIBRARY_PATH=/usr/local/netcdf/lib:$LD_LIBRARY_PATH \
    LIBRARY_PATH=/usr/local/netcdf/lib:$LIBRARY_PATH \
    PATH=/usr/local/netcdf/bin:$PATH
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://github.com/Unidata/netcdf-cxx4/archive/v4.3.1.tar.gz && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/v4.3.1.tar.gz -C /var/tmp -z && \
    cd /var/tmp/netcdf-cxx4-4.3.1 &&   ./configure --prefix=/usr/local/netcdf && \
    make -j$(nproc) && \
    make -j$(nproc) install && \
    rm -rf /var/tmp/netcdf-cxx4-4.3.1 /var/tmp/v4.3.1.tar.gz
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://github.com/Unidata/netcdf-fortran/archive/v4.5.3.tar.gz && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/v4.5.3.tar.gz -C /var/tmp -z && \
    cd /var/tmp/netcdf-fortran-4.5.3 &&   ./configure --prefix=/usr/local/netcdf && \
    make -j$(nproc) && \
    make -j$(nproc) install && \
    rm -rf /var/tmp/netcdf-fortran-4.5.3 /var/tmp/v4.5.3.tar.gz''')

    @ubuntu
    @docker
    def test_ldconfig(self):
        """ldconfig option"""
        n = netcdf(ldconfig=True, version='4.6.1', version_cxx='4.3.0',
                   version_fortran='4.4.4')
        self.assertEqual(str(n),
r'''# NetCDF version 4.6.1, NetCDF C++ version 4.3.0, NetCDF Fortran
# version 4.4.4
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        ca-certificates \
        file \
        libcurl4-openssl-dev \
        m4 \
        make \
        wget \
        zlib1g-dev && \
    rm -rf /var/lib/apt/lists/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://github.com/Unidata/netcdf-c/archive/v4.6.1.tar.gz && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/v4.6.1.tar.gz -C /var/tmp -z && \
    cd /var/tmp/netcdf-c-4.6.1 &&   ./configure --prefix=/usr/local/netcdf && \
    make -j$(nproc) && \
    make -j$(nproc) install && \
    echo "/usr/local/netcdf/lib" >> /etc/ld.so.conf.d/hpccm.conf && ldconfig && \
    rm -rf /var/tmp/netcdf-c-4.6.1 /var/tmp/v4.6.1.tar.gz
ENV CPATH=/usr/local/netcdf/include:$CPATH \
    LIBRARY_PATH=/usr/local/netcdf/lib:$LIBRARY_PATH \
    PATH=/usr/local/netcdf/bin:$PATH
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://github.com/Unidata/netcdf-cxx4/archive/v4.3.0.tar.gz && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/v4.3.0.tar.gz -C /var/tmp -z && \
    cd /var/tmp/netcdf-cxx4-4.3.0 &&   ./configure --prefix=/usr/local/netcdf && \
    make -j$(nproc) && \
    make -j$(nproc) install && \
    echo "/usr/local/netcdf/lib" >> /etc/ld.so.conf.d/hpccm.conf && ldconfig && \
    rm -rf /var/tmp/netcdf-cxx4-4.3.0 /var/tmp/v4.3.0.tar.gz
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://github.com/Unidata/netcdf-fortran/archive/v4.4.4.tar.gz && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/v4.4.4.tar.gz -C /var/tmp -z && \
    cd /var/tmp/netcdf-fortran-4.4.4 &&   ./configure --prefix=/usr/local/netcdf && \
    make -j$(nproc) && \
    make -j$(nproc) install && \
    echo "/usr/local/netcdf/lib" >> /etc/ld.so.conf.d/hpccm.conf && ldconfig && \
    rm -rf /var/tmp/netcdf-fortran-4.4.4 /var/tmp/v4.4.4.tar.gz''')

    @ubuntu
    @docker
    def test_runtime(self):
        """Runtime"""
        n = netcdf()
        r = n.runtime()
        self.assertEqual(r,
r'''# NetCDF
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        zlib1g && \
    rm -rf /var/lib/apt/lists/*
COPY --from=0 /usr/local/netcdf /usr/local/netcdf
ENV CPATH=/usr/local/netcdf/include:$CPATH \
    LD_LIBRARY_PATH=/usr/local/netcdf/lib:$LD_LIBRARY_PATH \
    LIBRARY_PATH=/usr/local/netcdf/lib:$LIBRARY_PATH \
    PATH=/usr/local/netcdf/bin:$PATH''')



# Copyright (c) 2018, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes

"""GDRCOPY building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import posixpath
from six.moves import shlex_quote
from distutils.version import LooseVersion

import hpccm.templates.envvars
import hpccm.templates.ldconfig

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.generic_build import generic_build
from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment
from hpccm.toolchain import toolchain

class gdrcopy(bb_base, hpccm.templates.envvars, hpccm.templates.ldconfig):
    """The `gdrcopy` building block builds and installs the user space
    library from the [gdrcopy](https://github.com/NVIDIA/gdrcopy)
    component.

    # Parameters

    annotate: Boolean flag to specify whether to include annotations
    (labels).  The default is False.

    environment: Boolean flag to specify whether the environment
    (`CPATH`, `LIBRARY_PATH`, and `LD_LIBRARY_PATH`) should be
    modified to include the gdrcopy. The default is True.

    ldconfig: Boolean flag to specify whether the gdrcopy library
    directory should be added dynamic linker cache.  If False, then
    `LD_LIBRARY_PATH` is modified to include the gdrcopy library
    directory. The default value is False.

    ospackages: List of OS packages to install prior to building.  The
    default values are `make` and `wget`.

    prefix: The top level install location.  The default value is
    `/usr/local/gdrcopy`.

    toolchain: The toolchain object.  This should be used if
    non-default compilers or other toolchain options are needed.  The
    default is empty.

    version: The version of gdrcopy source to download.  The default
    value is `2.2`.

    # Examples

    ```python
    gdrcopy(prefix='/opt/gdrcopy/2.1', version='2.1')
    ```

    """

    def __init__(self, **kwargs):
        """Initialize building block"""

        super(gdrcopy, self).__init__(**kwargs)

        # Parameters
        self.__baseurl = kwargs.pop('baseurl', 'https://github.com/NVIDIA/gdrcopy/archive')
        self.__ospackages = kwargs.pop('ospackages', ['make', 'wget'])
        self.__prefix = kwargs.pop('prefix', '/usr/local/gdrcopy')
        self.__toolchain = kwargs.pop('toolchain', toolchain())
        self.__version = kwargs.pop('version', '2.2')


        # Since gdrcopy does not use autotools or CMake, the toolchain
        # requires special handling.
        make_opts = vars(self.__toolchain)
        if 'CFLAGS' in make_opts:
            # CFLAGS is derived from COMMONCFLAGS, so rename.  See
            # https://github.com/NVIDIA/gdrcopy/blob/master/src/Makefile#L9
            make_opts['COMMONCFLAGS'] = make_opts.pop('CFLAGS')

        # Version 2.2 changed the flag to lowercase prefix and the lib directory
        if LooseVersion(self.__version) >= LooseVersion('2.2'):
            make_opts['prefix'] = self.__prefix
            libdir = 'lib'
        else:
            make_opts['PREFIX'] = self.__prefix        
            libdir = 'lib64'            

        make_opts_str = ' '.join(['{0}={1}'.format(key, shlex_quote(value))
                                  for key, value in sorted(make_opts.items())])

        # Setup the environment variables
        self.environment_variables['CPATH'] = '{}:$CPATH'.format(
            posixpath.join(self.__prefix, 'include'))
        self.environment_variables['LIBRARY_PATH'] = '{}:$LIBRARY_PATH'.format(
            posixpath.join(self.__prefix, libdir))
        if not self.ldconfig:
            self.environment_variables['LD_LIBRARY_PATH'] = '{}:$LD_LIBRARY_PATH'.format(
                posixpath.join(self.__prefix, libdir))


        # Setup build configuration
        self.__bb = generic_build(
            annotations={'version': self.__version},
            base_annotation=self.__class__.__name__,
            # Work around "install -D" issue on CentOS
            build=['mkdir -p {0}/include {0}/{1}'.format(self.__prefix, libdir),
                   'make {} lib lib_install'.format(make_opts_str)],
            comment=False,
            devel_environment=self.environment_variables,
            directory='gdrcopy-{}'.format(self.__version),
            libdir=libdir,
            prefix=self.__prefix,
            runtime_environment=self.environment_variables,
            url='{0}/v{1}.tar.gz'.format(self.__baseurl, self.__version),
            **kwargs)

        # Container instructions
        self += comment('GDRCOPY version {}'.format(self.__version))
        self += packages(ospackages=self.__ospackages)
        self += self.__bb

    def runtime(self, _from='0'):
        """Generate the set of instructions to install the runtime specific
        components from a build in a previous stage.

        # Examples

        ```python
        g = gdrcopy(...)
        Stage0 += g
        Stage1 += g.runtime()
        ```
        """
        self.rt += comment('GDRCOPY')
        self.rt += self.__bb.runtime(_from=_from)
        return str(self.rt)



"""EasyBuild container (https://github.com/easybuilders/easybuild)
   Set the user argument 'easyconfig' to specify the EasyConfig to
   build.  Otherwise, it will just build a base EasyBuild container
   image.

   Sample workflow:
$ hpccm.py --recipe recipes/easybuild.py --userarg easyconfig=GROMACS-2019.3-fosscuda-2019b.eb > Dockerfile.gromacs.eb
$ docker build -t gromacs.eb -f Dockerfile.gromacs.eb .
$ nvidia-docker run --rm -ti gromacs.eb bash -l
container:/tmp> module load GROMACS
...
"""
# pylint: disable=invalid-name, undefined-variable, used-before-assignment
import os

Stage0 += comment(__doc__, reformat=False)

Stage0 += baseimage(image='rockylinux:8')

Stage0 += shell(commands=['yum update -y rocky-release',
                          'rm -rf /var/cache/yum/*'])

# Base dependencies
Stage0 += python(python3=False)
Stage0 += gnu()
Stage0 += ofed()
Stage0 += packages(epel=True, powertools=True,
                   yum=['bzip2', 'diffutils', 'file', 'git', 'gzip',
                        'libtool', 'Lmod', 'make', 'openssh-clients',
                        'openssl-devel', 'patch', 'rsh', 'tar', 'unzip',
                        'which', 'xz'])

# Setup and install EasyBuild
Stage0 += pip(packages=['easybuild'], pip='pip2')
Stage0 += shell(commands=['useradd -m easybuild',
                          'mkdir -p /opt/easybuild',
                          'chown easybuild:easybuild /opt/easybuild'])

# Module environment
Stage0 += environment(variables={'MODULEPATH': '/opt/easybuild/modules/all:/home/easybuild/.local/easybuild/modules/all:$MODULEPATH'})

easyconfig = USERARG.get('easyconfig', None)
if easyconfig:
     # If the easyconfig is a file in the local build context, inject it
     # into the container image
     if os.path.isfile(easyconfig):
          Stage0 += copy(src=easyconfig, dest='/home/easybuild')

     Stage0 += shell(commands=['runuser easybuild -l -c "eb {} -r --installpath /opt/easybuild"'.format(easyconfig)])



# Copyright (c) 2018, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=invalid-name, too-few-public-methods, bad-continuation

"""Test cases for the cmake module"""

from __future__ import unicode_literals
from __future__ import print_function

import logging # pylint: disable=unused-import
import unittest

from helpers import aarch64, centos, docker, ubuntu, x86_64

from hpccm.building_blocks.cmake import cmake

class Test_cmake(unittest.TestCase):
    def setUp(self):
        """Disable logging output messages"""
        logging.disable(logging.ERROR)

    @x86_64
    @ubuntu
    @docker
    def test_defaults_ubuntu(self):
        """Default cmake building block"""
        c = cmake()
        self.assertEqual(str(c),
r'''# CMake version 3.22.2
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        make \
        wget && \
    rm -rf /var/lib/apt/lists/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://github.com/Kitware/CMake/releases/download/v3.22.2/cmake-3.22.2-linux-x86_64.sh && \
    mkdir -p /usr/local && \
    /bin/sh /var/tmp/cmake-3.22.2-linux-x86_64.sh --prefix=/usr/local && \
    rm -rf /var/tmp/cmake-3.22.2-linux-x86_64.sh
ENV PATH=/usr/local/bin:$PATH''')

    @x86_64
    @centos
    @docker
    def test_defaults_centos(self):
        """Default cmake building block"""
        c = cmake()
        self.assertEqual(str(c),
r'''# CMake version 3.22.2
RUN yum install -y \
        make \
        wget && \
    rm -rf /var/cache/yum/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://github.com/Kitware/CMake/releases/download/v3.22.2/cmake-3.22.2-linux-x86_64.sh && \
    mkdir -p /usr/local && \
    /bin/sh /var/tmp/cmake-3.22.2-linux-x86_64.sh --prefix=/usr/local && \
    rm -rf /var/tmp/cmake-3.22.2-linux-x86_64.sh
ENV PATH=/usr/local/bin:$PATH''')

    @x86_64
    @ubuntu
    @docker
    def test_eula(self):
        """3.20 binary naming"""
        c = cmake(version='3.20.0')
        self.assertEqual(str(c),
r'''# CMake version 3.20.0
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        make \
        wget && \
    rm -rf /var/lib/apt/lists/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://github.com/Kitware/CMake/releases/download/v3.20.0/cmake-3.20.0-linux-x86_64.sh && \
    mkdir -p /usr/local && \
    /bin/sh /var/tmp/cmake-3.20.0-linux-x86_64.sh --prefix=/usr/local && \
    rm -rf /var/tmp/cmake-3.20.0-linux-x86_64.sh
ENV PATH=/usr/local/bin:$PATH''')

    @x86_64
    @ubuntu
    @docker
    def test_eula(self):
        """Accept EULA"""
        c = cmake(eula=True, version='3.16.3')
        self.assertEqual(str(c),
r'''# CMake version 3.16.3
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        make \
        wget && \
    rm -rf /var/lib/apt/lists/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://github.com/Kitware/CMake/releases/download/v3.16.3/cmake-3.16.3-Linux-x86_64.sh && \
    mkdir -p /usr/local && \
    /bin/sh /var/tmp/cmake-3.16.3-Linux-x86_64.sh --prefix=/usr/local --skip-license && \
    rm -rf /var/tmp/cmake-3.16.3-Linux-x86_64.sh
ENV PATH=/usr/local/bin:$PATH''')

    @x86_64
    @ubuntu
    @docker
    def test_version(self):
        """Version option"""
        c = cmake(eula=True, version='3.10.3')
        self.assertEqual(str(c),
r'''# CMake version 3.10.3
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        make \
        wget && \
    rm -rf /var/lib/apt/lists/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://github.com/Kitware/CMake/releases/download/v3.10.3/cmake-3.10.3-Linux-x86_64.sh && \
    mkdir -p /usr/local && \
    /bin/sh /var/tmp/cmake-3.10.3-Linux-x86_64.sh --prefix=/usr/local --skip-license && \
    rm -rf /var/tmp/cmake-3.10.3-Linux-x86_64.sh
ENV PATH=/usr/local/bin:$PATH''')

    @x86_64
    @ubuntu
    @docker
    def test_defaults_ubuntu(self):
        """Cmake building block for old 32-bit versions"""
        c = cmake(eula=True, version='3.0.0')
        self.assertEqual(str(c),
r'''# CMake version 3.0.0
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        libc6-i386 \
        make \
        wget && \
    rm -rf /var/lib/apt/lists/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://github.com/Kitware/CMake/releases/download/v3.0.0/cmake-3.0.0-Linux-i386.sh && \
    mkdir -p /usr/local && \
    /bin/sh /var/tmp/cmake-3.0.0-Linux-i386.sh --prefix=/usr/local --skip-license && \
    rm -rf /var/tmp/cmake-3.0.0-Linux-i386.sh
ENV PATH=/usr/local/bin:$PATH''')

    @x86_64
    @centos
    @docker
    def test_defaults_ubuntu(self):
        """Cmake building block for old 32-bit versions"""
        c = cmake(eula=True, version='3.0.0')
        self.assertEqual(str(c),
r'''# CMake version 3.0.0
RUN yum install -y \
        glibc.i686 \
        make \
        wget && \
    rm -rf /var/cache/yum/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://github.com/Kitware/CMake/releases/download/v3.0.0/cmake-3.0.0-Linux-i386.sh && \
    mkdir -p /usr/local && \
    /bin/sh /var/tmp/cmake-3.0.0-Linux-i386.sh --prefix=/usr/local --skip-license && \
    rm -rf /var/tmp/cmake-3.0.0-Linux-i386.sh
ENV PATH=/usr/local/bin:$PATH''')

    @x86_64
    @centos
    @docker
    def test_source(self):
        """Source option"""
        c = cmake(eula=True, source=True, version='3.14.5')
        self.assertEqual(str(c),
r'''# CMake version 3.14.5
RUN yum install -y \
        make \
        openssl-devel \
        wget && \
    rm -rf /var/cache/yum/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://github.com/Kitware/CMake/releases/download/v3.14.5/cmake-3.14.5.tar.gz && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/cmake-3.14.5.tar.gz -C /var/tmp -z && \
    cd /var/tmp/cmake-3.14.5 && ./bootstrap --prefix=/usr/local --parallel=$(nproc) && \
    make -j$(nproc) && \
    make install && \
    rm -rf /var/tmp/cmake-3.14.5.tar.gz /var/tmp/cmake-3.14.5
ENV PATH=/usr/local/bin:$PATH''')

    @aarch64
    @centos
    @docker
    def test_aarch64(self):
        """Source option"""
        c = cmake(eula=True, version='3.14.5')
        self.assertEqual(str(c),
r'''# CMake version 3.14.5
RUN yum install -y \
        make \
        openssl-devel \
        wget && \
    rm -rf /var/cache/yum/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://github.com/Kitware/CMake/releases/download/v3.14.5/cmake-3.14.5.tar.gz && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/cmake-3.14.5.tar.gz -C /var/tmp -z && \
    cd /var/tmp/cmake-3.14.5 && ./bootstrap --prefix=/usr/local --parallel=$(nproc) && \
    make -j$(nproc) && \
    make install && \
    rm -rf /var/tmp/cmake-3.14.5.tar.gz /var/tmp/cmake-3.14.5
ENV PATH=/usr/local/bin:$PATH''')



# Use the generic OFED+UCX+OpenMPI recipe
hpccm.include('common.py')

# Build the OSU Micro-Benchmarks in the development stage
Stage0 += generic_autotools(
    build_environment={'CC': 'mpicc', 'CXX': 'mpicxx'},
    enable_cuda=True,
    prefix='/usr/local/osu',
    url='http://mvapich.cse.ohio-state.edu/download/mvapich/osu-micro-benchmarks-5.7.tar.gz',
    with_cuda='/usr/local/cuda')

# Copy the OSU Micro-Benchmark binaries into the deployment stage
Stage1 += copy(_from='devel', src='/usr/local/osu', dest='/usr/local/osu')

# Add the OSU Micro-Benchmarks to the default PATH
base_path = '/usr/local/osu/libexec/osu-micro-benchmarks'
Stage1 += environment(variables={'PATH': '{0}:{0}/mpi/collective:{0}/mpi/one-sided:{0}/mpi/pt2pt:{0}/mpi/startup:$PATH'.format(base_path)})



# Copyright (c) 2019, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=invalid-name, too-few-public-methods, bad-continuation

"""Test cases for the slurm_pmi2 module"""

from __future__ import unicode_literals
from __future__ import print_function

import logging # pylint: disable=unused-import
import unittest

from helpers import centos, docker, ubuntu, x86_64

from hpccm.building_blocks.slurm_pmi2 import slurm_pmi2

class Test_slurm_pmi2(unittest.TestCase):
    def setUp(self):
        """Disable logging output messages"""
        logging.disable(logging.ERROR)

    @x86_64
    @ubuntu
    @docker
    def test_defaults_ubuntu(self):
        """Default slurm_pmi2 building block"""
        p = slurm_pmi2()
        self.assertEqual(str(p),
r'''# SLURM PMI2 version 21.08.8
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        bzip2 \
        file \
        make \
        perl \
        tar \
        wget && \
    rm -rf /var/lib/apt/lists/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://download.schedmd.com/slurm/slurm-21.08.8.tar.bz2 && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/slurm-21.08.8.tar.bz2 -C /var/tmp -j && \
    cd /var/tmp/slurm-21.08.8 &&   ./configure --prefix=/usr/local/slurm-pmi2 && \
    cd /var/tmp/slurm-21.08.8 && \
    make -C contribs/pmi2 install && \
    rm -rf /var/tmp/slurm-21.08.8 /var/tmp/slurm-21.08.8.tar.bz2''')

    @x86_64
    @ubuntu
    @docker
    def test_ldconfig(self):
        """ldconfig option"""
        p = slurm_pmi2(ldconfig=True, version='20.02.7')
        self.assertEqual(str(p),
r'''# SLURM PMI2 version 20.02.7
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        bzip2 \
        file \
        make \
        perl \
        tar \
        wget && \
    rm -rf /var/lib/apt/lists/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://download.schedmd.com/slurm/slurm-20.02.7.tar.bz2 && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/slurm-20.02.7.tar.bz2 -C /var/tmp -j && \
    cd /var/tmp/slurm-20.02.7 &&   ./configure --prefix=/usr/local/slurm-pmi2 && \
    cd /var/tmp/slurm-20.02.7 && \
    make -C contribs/pmi2 install && \
    echo "/usr/local/slurm-pmi2/lib" >> /etc/ld.so.conf.d/hpccm.conf && ldconfig && \
    rm -rf /var/tmp/slurm-20.02.7 /var/tmp/slurm-20.02.7.tar.bz2''')

    @x86_64
    @ubuntu
    @docker
    def test_runtime(self):
        """Runtime"""
        p = slurm_pmi2()
        r = p.runtime()
        self.assertEqual(r,
r'''# SLURM PMI2
COPY --from=0 /usr/local/slurm-pmi2 /usr/local/slurm-pmi2''')



# Copyright (c) 2018, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=invalid-name, too-few-public-methods

"""Test cases for the config module"""

from __future__ import unicode_literals
from __future__ import print_function

from distutils.version import StrictVersion
import logging # pylint: disable=unused-import
import unittest

from helpers import bash, broadwell, centos, docker, icelake, singularity, thunderx2, ubuntu, zen2

import hpccm.config

class Test_config(unittest.TestCase):
    def setUp(self):
        """Disable logging output messages"""
        logging.disable(logging.ERROR)

    @singularity
    def test_set_container_format_docker(self):
        """Set container format to Docker"""
        hpccm.config.set_container_format('docker')
        self.assertEqual(hpccm.config.g_ctype, hpccm.container_type.DOCKER)

    @docker
    def test_set_container_format_singularity(self):
        """Set container format to Singularity"""
        hpccm.config.set_container_format('singularity')
        self.assertEqual(hpccm.config.g_ctype, hpccm.container_type.SINGULARITY)

    @docker
    def test_set_container_format_invalid(self):
        """Set container format to invalid value"""
        with self.assertRaises(RuntimeError):
            hpccm.config.set_container_format('invalid')

    @docker
    def test_set_linux_distro_ubuntu(self):
        """Set Linux distribution to Ubuntu"""
        hpccm.config.set_linux_distro('ubuntu')
        self.assertEqual(hpccm.config.g_linux_distro,
                         hpccm.linux_distro.UBUNTU)
        self.assertEqual(hpccm.config.g_linux_version, StrictVersion('16.04'))

        hpccm.config.set_linux_distro('ubuntu16')
        self.assertEqual(hpccm.config.g_linux_distro,
                         hpccm.linux_distro.UBUNTU)
        self.assertEqual(hpccm.config.g_linux_version, StrictVersion('16.04'))

        hpccm.config.set_linux_distro('ubuntu18')
        self.assertEqual(hpccm.config.g_linux_distro,
                         hpccm.linux_distro.UBUNTU)
        self.assertEqual(hpccm.config.g_linux_version, StrictVersion('18.04'))

    @docker
    def test_set_linux_distro_centos(self):
        """Set Linux distribution to CentOS"""
        hpccm.config.set_linux_distro('centos')
        self.assertEqual(hpccm.config.g_linux_distro,
                         hpccm.linux_distro.CENTOS)
        self.assertEqual(hpccm.config.g_linux_version, StrictVersion('7.0'))

        hpccm.config.set_linux_distro('centos7')
        self.assertEqual(hpccm.config.g_linux_distro,
                         hpccm.linux_distro.CENTOS)
        self.assertEqual(hpccm.config.g_linux_version, StrictVersion('7.0'))

        hpccm.config.set_linux_distro('centos8')
        self.assertEqual(hpccm.config.g_linux_distro,
                         hpccm.linux_distro.CENTOS)
        self.assertEqual(hpccm.config.g_linux_version, StrictVersion('8.0'))

    @docker
    def test_set_linux_distro_rhel(self):
        """Set Linux distribution to RHEL"""
        hpccm.config.set_linux_distro('rhel')
        self.assertEqual(hpccm.config.g_linux_distro,
                         hpccm.linux_distro.RHEL)
        self.assertEqual(hpccm.config.g_linux_version, StrictVersion('7.0'))

        hpccm.config.set_linux_distro('rhel7')
        self.assertEqual(hpccm.config.g_linux_distro,
                         hpccm.linux_distro.RHEL)
        self.assertEqual(hpccm.config.g_linux_version, StrictVersion('7.0'))

        hpccm.config.set_linux_distro('rhel8')
        self.assertEqual(hpccm.config.g_linux_distro,
                         hpccm.linux_distro.RHEL)
        self.assertEqual(hpccm.config.g_linux_version, StrictVersion('8.0'))

    @docker
    def test_set_linux_distro_rockylinux(self):
        """Set Linux distribution to Rocky Linux"""
        hpccm.config.set_linux_distro('rockylinux8')
        self.assertEqual(hpccm.config.g_linux_distro,
                         hpccm.linux_distro.ROCKYLINUX)
        self.assertEqual(hpccm.config.g_linux_version, StrictVersion('8.0'))

    @docker
    def test_set_linux_distro_invalid(self):
        """Set Linux distribution to an invalid value"""
        hpccm.config.set_linux_distro('invalid')
        self.assertEqual(hpccm.config.g_linux_distro,
                         hpccm.linux_distro.UBUNTU)
        self.assertEqual(hpccm.config.g_linux_version, StrictVersion('16.04'))

    @singularity
    def test_set_singularity_version(self):
        """Set Singularity version"""
        hpccm.config.set_singularity_version('10.0')
        self.assertEqual(hpccm.config.g_singularity_version,
                         StrictVersion('10.0'))

    @docker
    def test_set_cpu_architecture_aarch64(self):
        """Set CPU architecture to ARM"""
        hpccm.config.set_cpu_architecture('aarch64')
        self.assertEqual(hpccm.config.g_cpu_arch, hpccm.cpu_arch.AARCH64)
        self.assertEqual(hpccm.config.get_cpu_architecture(), 'aarch64')

    @docker
    def test_set_cpu_architecture_arm(self):
        """Set CPU architecture to ARM"""
        hpccm.config.set_cpu_architecture('arm')
        self.assertEqual(hpccm.config.g_cpu_arch, hpccm.cpu_arch.AARCH64)

    @docker
    def test_set_cpu_architecture_arm64v8(self):
        """Set CPU architecture to ARM"""
        hpccm.config.set_cpu_architecture('arm64v8')
        self.assertEqual(hpccm.config.g_cpu_arch, hpccm.cpu_arch.AARCH64)

    @ubuntu
    @docker
    def test_set_cpu_architecture_ppc64le_ubuntu(self):
        """Set CPU architecture to POWER"""
        hpccm.config.set_cpu_architecture('ppc64le')
        self.assertEqual(hpccm.config.g_cpu_arch, hpccm.cpu_arch.PPC64LE)
        self.assertEqual(hpccm.config.get_cpu_architecture(), 'ppc64el')

    @centos
    @docker
    def test_set_cpu_architecture_ppc64le_centos(self):
        """Set CPU architecture to POWER"""
        hpccm.config.set_cpu_architecture('ppc64le')
        self.assertEqual(hpccm.config.g_cpu_arch, hpccm.cpu_arch.PPC64LE)
        self.assertEqual(hpccm.config.get_cpu_architecture(), 'ppc64le')

    @docker
    def test_set_cpu_architecture_power(self):
        """Set CPU architecture to POWER"""
        hpccm.config.set_cpu_architecture('power')
        self.assertEqual(hpccm.config.g_cpu_arch, hpccm.cpu_arch.PPC64LE)

    @docker
    def test_set_cpu_architecture_x86_64(self):
        """Set CPU architecture to x86_64"""
        hpccm.config.set_cpu_architecture('x86_64')
        self.assertEqual(hpccm.config.g_cpu_arch, hpccm.cpu_arch.X86_64)
        self.assertEqual(hpccm.config.get_cpu_architecture(), 'x86_64')

    @docker
    def test_set_cpu_architecture_amd64(self):
        """Set CPU architecture to x86_64"""
        hpccm.config.set_cpu_architecture('amd64')
        self.assertEqual(hpccm.config.g_cpu_arch, hpccm.cpu_arch.X86_64)

    @docker
    def test_set_cpu_architecture_x86(self):
        """Set CPU architecture to x86_64"""
        hpccm.config.set_cpu_architecture('x86')
        self.assertEqual(hpccm.config.g_cpu_arch, hpccm.cpu_arch.X86_64)

    @docker
    def test_set_cpu_architecture_invalid(self):
        """Set CPU architecture to invalid value"""
        hpccm.config.set_cpu_architecture('invalid')
        self.assertEqual(hpccm.config.g_cpu_arch, hpccm.cpu_arch.X86_64)

    @bash
    def test_get_format_bash(self):
        """Get container format"""
        self.assertEqual(hpccm.config.get_format(), 'bash')

    @docker
    def test_get_format_docker(self):
        """Get container format"""
        self.assertEqual(hpccm.config.get_format(), 'docker')

    @singularity
    def test_get_format_singularity(self):
        """Get container format"""
        self.assertEqual(hpccm.config.get_format(), 'singularity')

    def test_set_working_directory(self):
        """Set working directory"""
        # save default value in order to switch back later
        default_wd = hpccm.config.g_wd

        hpccm.config.set_working_directory('/a/b')
        self.assertEqual(hpccm.config.g_wd, '/a/b')

        # reset to the default working directory
        hpccm.config.set_working_directory(default_wd)

    def test_set_cpu_target(self):
        """Set CPU optimization target"""
        # save default value in order to switch back later
        default_cpu_target = hpccm.config.g_cpu_target

        hpccm.config.set_cpu_target('broadwell')
        self.assertEqual(hpccm.config.g_cpu_target, 'broadwell')

        # reset to the default cpu optimization target
        hpccm.config.set_cpu_target(default_cpu_target)

    @thunderx2
    def test_get_cpu_optimization_flags(self):
        """Get CPU optimization flags"""
        flags = hpccm.config.get_cpu_optimization_flags('gcc')
        self.assertEqual(flags, '-mcpu=thunderx2t99')

    @icelake
    def test_get_cpu_optimization_flags_old_compiler(self):
        """Get CPU optimization flags"""
        flags = hpccm.config.get_cpu_optimization_flags('gcc', version='4.8.5')
        self.assertEqual(flags, None)

    @zen2
    def test_test_feature_flag(self):
        """Test CPU feature flags"""
        self.assertTrue(hpccm.config.test_cpu_feature_flag('avx2'))
        self.assertFalse(hpccm.config.test_cpu_feature_flag('foo'))



# Copyright (c) 2020, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes

"""NVIDIA Nsight Compute building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function
import os

from distutils.version import StrictVersion
import posixpath

import hpccm.config
import hpccm.templates.envvars

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.packages import packages
from hpccm.building_blocks.generic_build import generic_build
from hpccm.common import cpu_arch, linux_distro
from hpccm.primitives.comment import comment
from hpccm.primitives.environment import environment

class nsight_compute(bb_base, hpccm.templates.envvars):
    """The `nsight_compute` building block downloads and installs the
    [NVIDIA Nsight Compute
    profiler]](https://developer.nvidia.com/nsight-compute).

    # Parameters

    eula: Required, by setting this value to `True`, you agree to the
    Nsight Compute End User License Agreement that is displayed when
    running the installer interactively.  The default value is
    `False`.

    ospackages: List of OS packages to install prior to building.
    When using a runfile, the default values are `perl` for Ubuntu and
    `perl` and `perl-Env` for RHEL-based Linux distributions.
    Otherwise, the default values are `apt-transport-https`,
    `ca-certificates`, `gnupg`, and `wget` for Ubuntu and an empty
    list for RHEL-based Linux distributions.

    prefix: The top level install prefix. The default value is
    `/usr/local/NVIDIA-Nsight-Compute`.  This parameter is ignored
    unless `runfile` is set.

    runfile: Path or URL to NSight Compute's `.run` file relative to the
    local build context. The default value is empty.

    version: the version of Nsight Compute to install.  Note when
    `runfile` is set this parameter is ignored.  The default value is
    `2022.1.1`.

    # Examples

    ```python
    nsight_compute(version='2020.2.1')
    ```

    ```python
    nsight_compute(eula=True, runfile='nsight-compute-linux-2020.2.0.18-28964561.run')
    ```

    """

    def __init__(self, **kwargs):
        """Initialize building block"""

        super(nsight_compute, self).__init__(**kwargs)

        self.__arch_label = ''   # Filled in __cpu_arch
        self.__distro_label = ''     # Filled in by __distro
        self.__eula = kwargs.get('eula', False)
        self.__ospackages = kwargs.get('ospackages', [])
        self.__prefix = kwargs.get('prefix',
                                   '/usr/local/NVIDIA-Nsight-Compute')
        self.__runfile = kwargs.get('runfile', None)
        self.__version = kwargs.get('version', '2022.1.1')
        self.__wd = kwargs.get('wd', posixpath.join(
            hpccm.config.g_wd, 'nsight_compute')) # working directory

        # Set the Linux distribution specific parameters
        self.__distro()

        # Disables deployment of section files to prevent warning
        # when there is no home or home is read-only:
        self.environment_variables[
            'NV_COMPUTE_PROFILER_DISABLE_STOCK_FILE_DEPLOYMENT'
        ] = '1'

        if self.__runfile:
            # Runfile based installation
            if not self.__eula:
                raise RuntimeError('Nsight Compute EULA was not accepted.')

            self.__instructions_runfile()

        else:
            # Package repository based installation

            # Set the CPU architecture specific parameters
            self.__cpu_arch()

            # Fill in container instructions
            self.__instructions_repository()

    def __cpu_arch(self):
        """Based on the CPU architecture, set values accordingly.  A user
        specified value overrides any defaults."""

        if hpccm.config.g_cpu_arch == cpu_arch.AARCH64:
            self.__arch_label = 'arm64'
        elif hpccm.config.g_cpu_arch == cpu_arch.PPC64LE:
            if hpccm.config.g_linux_distro == linux_distro.UBUNTU:
                self.__arch_label = 'ppc64el'
            else:
                self.__arch_label = 'ppc64le'
        elif hpccm.config.g_cpu_arch == cpu_arch.X86_64:
            if hpccm.config.g_linux_distro == linux_distro.UBUNTU:
                self.__arch_label = 'amd64'
            else:
                self.__arch_label = 'x86_64'
        else: # pragma: no cover
            raise RuntimeError('Unknown CPU architecture')

    def __distro(self):
        """Based on the Linux distribution, set values accordingly.  A user
        specified value overrides any defaults."""

        if hpccm.config.g_linux_distro == linux_distro.UBUNTU:
            if not self.__ospackages:
                if self.__runfile:
                    self.__ospackages = ['perl', 'wget']
                else:
                    self.__ospackages = ['apt-transport-https',
                                         'ca-certificates', 'gnupg', 'wget']

            if hpccm.config.g_linux_version >= StrictVersion('20.04'):
                self.__distro_label = 'ubuntu2004'
            elif hpccm.config.g_linux_version >= StrictVersion('18.0'):
                self.__distro_label = 'ubuntu1804'
            else:
                self.__distro_label = 'ubuntu1604'

        elif hpccm.config.g_linux_distro == linux_distro.CENTOS:
            if not self.__ospackages:
                if self.__runfile:
                    self.__ospackages = ['perl', 'perl-Env', 'wget']

            if hpccm.config.g_linux_version >= StrictVersion('8.0'):
                self.__distro_label = 'rhel8'
            else:
                self.__distro_label = 'rhel7'

        else: # pragma: no cover
            raise RuntimeError('Unknown Linux distribution')

    def __instructions_repository(self):
        """Fill in container instructions"""

        self += comment('NVIDIA Nsight Compute {}'.format(self.__version))

        if self.__ospackages:
            self += packages(ospackages=self.__ospackages)

        self += packages(
            apt_keys=['https://developer.download.nvidia.com/devtools/repos/{0}/{1}/nvidia.pub'.format(self.__distro_label, self.__arch_label)],
            apt_repositories=['deb https://developer.download.nvidia.com/devtools/repos/{0}/{1}/ /'.format(self.__distro_label, self.__arch_label)],
            # https://github.com/NVIDIA/hpc-container-maker/issues/367
            force_add_repo=True,
            ospackages=['nsight-compute-{}'.format(self.__version)],
            yum_keys=['https://developer.download.nvidia.com/devtools/repos/{0}/{1}/nvidia.pub'.format(self.__distro_label, self.__arch_label)],
            yum_repositories=['https://developer.download.nvidia.com/devtools/repos/{0}/{1}'.format(self.__distro_label, self.__arch_label)])

        # The distro packages do not link nsight-compute binaries to /usr/local/bin
        self.environment_variables['PATH'] = '/opt/nvidia/nsight-compute/{}:$PATH'.format(self.__version)
        self += environment(variables=self.environment_step())

    def __instructions_runfile(self):
        """Fill in container instructions"""

        pkg = os.path.basename(self.__runfile)

        install_cmds = [
            'sh ./{} --nox11 -- -noprompt -targetpath={}'.format(
                pkg, self.__prefix)
        ]

        # Commands needed to predeploy target-specific files. When
        # connecting through the GUI on another machine to the
        # container, this removes the need to copy the files over.
        install_cmds += [
            'mkdir -p /tmp/var/target',
            'ln -sf {}/target/* /tmp/var/target'.format(self.__prefix),
            'ln -sf {}/sections /tmp/var/'.format(self.__prefix),
            'chmod -R a+w /tmp/var'
        ]

        kwargs = {}
        if self.__runfile.strip().startswith(('http://', 'https://')):
            kwargs['url'] = self.__runfile
        else:
            kwargs['package'] = self.__runfile

        self.__bb = generic_build(
            annotations={'runfile': pkg},
            base_annotation=self.__class__.__name__,
            comment = False,
            devel_environment={'PATH': '{}:$PATH'.format(self.__prefix)},
            directory=self.__wd,
            install=install_cmds,
            unpack=False,
            wd=self.__wd,
            **kwargs
        )

        self += comment('NVIDIA Nsight Compute {}'.format(pkg), reformat=False)
        self += packages(ospackages=self.__ospackages)
        self += self.__bb
        self += environment(variables=self.environment_variables)



# Copyright (c) 2019, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes

"""MPICH building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import posixpath
import re
from copy import copy as _copy

import hpccm.config
import hpccm.templates.envvars
import hpccm.templates.ldconfig

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.generic_autotools import generic_autotools
from hpccm.building_blocks.packages import packages
from hpccm.common import linux_distro
from hpccm.primitives.comment import comment
from hpccm.toolchain import toolchain

class mpich(bb_base, hpccm.templates.envvars, hpccm.templates.ldconfig):
    """The `mpich` building block configures, builds, and installs the
    [MPICH](https://www.mpich.org) component.

    As a side effect, a toolchain is created containing the MPI
    compiler wrappers.  The tool can be passed to other operations
    that want to build using the MPI compiler wrappers.

    # Parameters

    annotate: Boolean flag to specify whether to include annotations
    (labels).  The default is False.

    check: Boolean flag to specify whether the `make check` and `make
    testing` steps should be performed.  The default is False.

    configure_opts: List of options to pass to `configure`.  The
    default is an empty list.

    disable_FEATURE: Flags to control disabling features when
    configuring.  For instance, `disable_foo=True` maps to
    `--disable-foo`.  Underscores in the parameter name are converted
    to dashes.

    enable_FEATURE[=ARG]: Flags to control enabling features when
    configuring.  For instance, `enable_foo=True` maps to
    `--enable-foo` and `enable_foo='yes'` maps to `--enable-foo=yes`.
    Underscores in the parameter name are converted to dashes.

    environment: Boolean flag to specify whether the environment
    (`LD_LIBRARY_PATH` and `PATH`) should be modified to include
    MPICH. The default is True.

    ldconfig: Boolean flag to specify whether the MPICH library
    directory should be added dynamic linker cache.  If False, then
    `LD_LIBRARY_PATH` is modified to include the MPICH library
    directory. The default value is False.

    ospackages: List of OS packages to install prior to configuring
    and building.  For Ubuntu, the default values are `file`, `gzip`,
    `make`, `openssh-client`, `perl`, `tar`, and `wget`.  For
    RHEL-based Linux distributions, the default values are `file`,
    `gzip`, `make`, `openssh-clients`, `perl`, `tar`, and `wget`.

    prefix: The top level install location.  The default value is
    `/usr/local/mpich`.

    toolchain: The toolchain object.  This should be used if
    non-default compilers or other toolchain options are needed.  The
    default is empty.

    version: The version of MPICH source to download.  The default
    value is `3.3.2`.

    with_PACKAGE[=ARG]: Flags to control optional packages when
    configuring.  For instance, `with_foo=True` maps to `--with-foo`
    and `with_foo='/usr/local/foo'` maps to
    `--with-foo=/usr/local/foo`.  Underscores in the parameter name
    are converted to dashes.

    without_PACKAGE: Flags to control optional packages when
    configuring.  For instance `without_foo=True` maps to
    `--without-foo`.  Underscores in the parameter name are converted
    to dashes.

    # Examples

    ```python
    mpich(prefix='/opt/mpich/3.3', version='3.3')
    ```

    """

    def __init__(self, **kwargs):
        """Initialize building block"""

        super(mpich, self).__init__(**kwargs)

        self.__baseurl = kwargs.pop('baseurl',
                                    'https://www.mpich.org/static/downloads')
        self.__check = kwargs.pop('check', False)
        self.__configure_opts = kwargs.pop('configure_opts', [])
        self.__ospackages = kwargs.pop('ospackages', [])
        self.__prefix = kwargs.pop('prefix', '/usr/local/mpich')
        self.__runtime_ospackages = [] # Filled in by __distro()
        # Input toolchain, i.e., what to use when building
        self.__toolchain = kwargs.pop('toolchain', toolchain())
        self.__version = kwargs.pop('version', '3.3.2')

        # Output toolchain
        self.toolchain = toolchain(CC='mpicc', CXX='mpicxx', F77='mpif77',
                                   F90='mpif90', FC='mpifort')

        # Set the configuration options
        self.__configure()

        # Set the Linux distribution specific parameters
        self.__distro()

        # Set the environment variables
        self.environment_variables['PATH'] = '{}:$PATH'.format(
            posixpath.join(self.__prefix, 'bin'))
        if not self.ldconfig:
            self.environment_variables['LD_LIBRARY_PATH'] = '{}:$LD_LIBRARY_PATH'.format(posixpath.join(self.__prefix, 'lib'))

        # Setup build configuration
        self.__bb = generic_autotools(
            annotations={'version': self.__version},
            base_annotation=self.__class__.__name__,
            check=self.__check,
            comment=False,
            configure_opts=self.__configure_opts,
            devel_environment=self.environment_variables,
            # Run test suite (must be after install)
            postinstall=['cd {0}/mpich-{1}'.format(hpccm.config.g_wd,
                                                   self.__version),
                         'RUNTESTS_SHOWPROGRESS=1 make testing'] if self.__check else None,
            prefix=self.__prefix,
            runtime_environment=self.environment_variables,
            toolchain=self.__toolchain,
            url='{0}/{1}/mpich-{1}.tar.gz'.format(self.__baseurl,
                                                  self.__version),
            **kwargs)

        # Container instructions
        self += comment('MPICH version {}'.format(self.__version))
        self += packages(ospackages=self.__ospackages)
        self += self.__bb

    def __configure(self):
        """Setup configure options based on user parameters"""

        # Create a copy of hte toolchain so that it can be modified
        # without impacting the original
        self.__toolchain = _copy(self.__toolchain)

        # MPICH does not accept F90
        self.__toolchain.F90 = ''

        # Workaround issue with the PGI compiler
        # https://lists.mpich.org/pipermail/discuss/2017-July/005235.html
        if self.__toolchain.CC and re.match('.*pgcc', self.__toolchain.CC):
            self.__configure_opts.append('--disable-fast')

    def __distro(self):
        """Based on the Linux distribution, set values accordingly.  A user
        specified value overrides any defaults."""

        if hpccm.config.g_linux_distro == linux_distro.UBUNTU:
            if not self.__ospackages:
                self.__ospackages = ['file', 'gzip', 'make', 'openssh-client',
                                     'perl', 'tar', 'wget']
            self.__runtime_ospackages = ['openssh-client']
        elif hpccm.config.g_linux_distro == linux_distro.CENTOS:
            if not self.__ospackages:
                self.__ospackages = ['file', 'gzip', 'make',
                                     'openssh-clients', 'perl', 'tar', 'wget']
            self.__runtime_ospackages = ['openssh-clients']
        else: # pragma: no cover
            raise RuntimeError('Unknown Linux distribution')

    def runtime(self, _from='0'):
        """Generate the set of instructions to install the runtime specific
        components from a build in a previous stage.

        # Examples
        ```python
        m = mpich(...)
        Stage0 += m
        Stage1 += m.runtime()
        ```
        """
        self.rt += comment('MPICH')
        self.rt += packages(ospackages=self.__runtime_ospackages)
        self.rt += self.__bb.runtime(_from=_from)
        return str(self.rt)



# Copyright (c) 2018, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes

"""AmgX building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import posixpath

import hpccm.config
import hpccm.templates.envvars
import hpccm.templates.ldconfig

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.generic_cmake import generic_cmake
from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment

class amgx(bb_base, hpccm.templates.envvars, hpccm.templates.ldconfig):
    """The `amgx` building block downloads, configures, builds, and
    installs the [AMGX](https://developer.nvidia.com/amgx) component.

    The [CMake](#cmake) building block should be installed prior to
    this building block.

    Installing an MPI building block before this one is optional and
    will build the [AMGX](https://developer.nvidia.com/amgx) library
    with MPI support.  Some Eigensolvers make use of the MAGMA and/or
    MKL libraries and are only available if the paths to these
    libraries is specified as shown below in the cmake_opts.

    # Parameters

    annotate: Boolean flag to specify whether to include annotations (labels).
    The default is False.

    branch: The git branch to clone.  AMGX releases are tagged, that
    is, specifying `branch='v2.1.0'` downloads a particular AMGX
    version.  The default is `master`.

    cmake_opts: List of options to pass to `cmake`.  The default value is an empty list.  See the ["Building"](https://github.com/NVIDIA/AMGX#-building) section of the AMGX documentation of the specified library version for more details.  Some options are `CMAKE_NO_MPI:Boolean` (default=`False`) - build without MPI support even if the `FindMPI` script finds an MPI library.  `AMGX_NO_RPATH:Boolean` (default=`False`) - by default CMake adds `-rpath` flags to binaries, this option disables that.  `MKL_ROOT_DIR:String`, `MAGMA_ROOT_DIR:String` - MAGMA/MKL are used to accelerate some of the Eigensolvers.  These solvers will return "error 'not supported'" if AMGX was not build with MKL/MAGMA support.

    commit: The git commit to clone.  The default is empty and uses
    the latest commit on the selected branch of the repository.

    directory: Build from an unpackaged source directory relative to
    the local build context instead of fetching AMGX sources from a
    git repository.  This option is incompatible with
    `repository`/`branch`/ `commit`.  The default is `None`.

    ospackages: List of OS packages to install prior to downloading,
    configuring, and building.  The default value is `[git]`.

    prefix: The top level install location.  The default is
    `/usr/local/amgx`.

    repository: The git repository to clone.  The default is `https://github.com/NVIDIA/AMGX`.

    toolchain: The toolchain object.  This should be used if
    non-default compilers or other toolchain options are needed.  The
    default is empty.

    # Examples

    ```python
    amgx(branch='v2.1.0')
    ```

    """

    def __init__(self, **kwargs):
        """Initialize building block"""
        super(amgx, self).__init__(**kwargs)

        self.__branch = kwargs.pop('branch', 'master')
        self.__cmake_opts = kwargs.pop('cmake_opts', [])
        self.__ospackages = kwargs.pop('ospackages', ['git', 'make'])
        self.__prefix = kwargs.pop('prefix', '/usr/local/amgx')
        self.__repository = kwargs.pop('repository', 'https://github.com/NVIDIA/amgx')

        # Set the environment
        self.environment_variables['CPATH'] = '{}:$CPATH'.format(
            posixpath.join(self.__prefix, 'include'))
        self.environment_variables['LIBRARY_PATH'] = '{}:$LIBRARY_PATH'.format(
            posixpath.join(self.__prefix, 'lib'))
        if not self.ldconfig:
            self.environment_variables['LD_LIBRARY_PATH'] = '{}:$LD_LIBRARY_PATH'.format(posixpath.join(self.__prefix, 'lib'))

        # Setup build configuration
        self.__bb = generic_cmake(
            annotations={'branch': self.__branch},
            base_annotation=self.__class__.__name__,
            branch=self.__branch,
            comment=False,
            cmake_opts=self.__cmake_opts,
            devel_environment=self.environment_variables,
            prefix=self.__prefix,
            runtime_environment=self.environment_variables,
            repository=self.__repository,
            **kwargs)

        # Container instructions
        self += comment('AMGX branch {}'.format(self.__branch))
        self += packages(ospackages=self.__ospackages)
        self += self.__bb


    def runtime(self, _from='0'):
        """Generate the set of instructions to install the runtime specific
        components from a build in a previous stage.

        # Examples

        ```python
        a = amgx(...)
        Stage0 += a
        Stage1 += a.runtime()
        ```
        """
        self.rt += comment('AMGX')
        self.rt += self.__bb.runtime(_from=_from)
        return str(self.rt)



# Copyright (c) 2019, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=invalid-name, too-few-public-methods, bad-continuation

"""Test cases for the generic_cmake module"""

from __future__ import unicode_literals
from __future__ import print_function

import logging # pylint: disable=unused-import
import unittest

from helpers import centos, docker, ubuntu

from hpccm.building_blocks.generic_cmake import generic_cmake
from hpccm.toolchain import toolchain

class Test_generic_cmake(unittest.TestCase):
    def setUp(self):
        """Disable logging output messages"""
        logging.disable(logging.ERROR)

    @ubuntu
    @docker
    def test_defaults_ubuntu(self):
        """Default generic_cmake building block"""
        g = generic_cmake(
            cmake_opts=['-D CMAKE_BUILD_TYPE=Release',
                        '-D CUDA_TOOLKIT_ROOT_DIR=/usr/local/cuda',
                        '-D GMX_BUILD_OWN_FFTW=ON',
                        '-D GMX_GPU=ON',
                        '-D GMX_MPI=OFF',
                        '-D GMX_OPENMP=ON',
                        '-D GMX_PREFER_STATIC_LIBS=ON',
                        '-D MPIEXEC_PREFLAGS=--allow-run-as-root'],
            directory='gromacs-2018.2',
            prefix='/usr/local/gromacs',
            url='https://github.com/gromacs/gromacs/archive/v2018.2.tar.gz')
        self.assertEqual(str(g),
r'''# https://github.com/gromacs/gromacs/archive/v2018.2.tar.gz
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://github.com/gromacs/gromacs/archive/v2018.2.tar.gz && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/v2018.2.tar.gz -C /var/tmp -z && \
    mkdir -p /var/tmp/gromacs-2018.2/build && cd /var/tmp/gromacs-2018.2/build && cmake -DCMAKE_INSTALL_PREFIX=/usr/local/gromacs -D CMAKE_BUILD_TYPE=Release -D CUDA_TOOLKIT_ROOT_DIR=/usr/local/cuda -D GMX_BUILD_OWN_FFTW=ON -D GMX_GPU=ON -D GMX_MPI=OFF -D GMX_OPENMP=ON -D GMX_PREFER_STATIC_LIBS=ON -D MPIEXEC_PREFLAGS=--allow-run-as-root /var/tmp/gromacs-2018.2 && \
    cmake --build /var/tmp/gromacs-2018.2/build --target all -- -j$(nproc) && \
    cmake --build /var/tmp/gromacs-2018.2/build --target install -- -j$(nproc) && \
    rm -rf /var/tmp/gromacs-2018.2 /var/tmp/v2018.2.tar.gz''')

    @ubuntu
    @docker
    def test_no_url(self):
        """missing url"""
        with self.assertRaises(RuntimeError):
            g = generic_cmake()

    @ubuntu
    @docker
    def test_both_repository_and_url(self):
        """both repository and url"""
        with self.assertRaises(RuntimeError):
            g = generic_cmake(repository='foo', url='bar')

    @ubuntu
    @docker
    def test_invalid_package(self):
        """invalid package url"""
        with self.assertRaises(RuntimeError):
            g = generic_cmake(url='https://foo/bar.sh')

    @ubuntu
    @docker
    def test_package(self):
        """local package"""
        g = generic_cmake(
            cmake_opts=['-D CMAKE_BUILD_TYPE=Release',
                        '-D CUDA_TOOLKIT_ROOT_DIR=/usr/local/cuda',
                        '-D GMX_BUILD_OWN_FFTW=ON',
                        '-D GMX_GPU=ON',
                        '-D GMX_MPI=OFF',
                        '-D GMX_OPENMP=ON',
                        '-D GMX_PREFER_STATIC_LIBS=ON',
                        '-D MPIEXEC_PREFLAGS=--allow-run-as-root',
                        '-D REGRESSIONTEST_DOWNLOAD=ON'],
            directory='gromacs-2018.2',
            package='gromacs/v2018.2.tar.gz',
            prefix='/usr/local/gromacs')
        self.assertEqual(str(g),
r'''# gromacs/v2018.2.tar.gz
COPY gromacs/v2018.2.tar.gz /var/tmp/v2018.2.tar.gz
RUN mkdir -p /var/tmp && tar -x -f /var/tmp/v2018.2.tar.gz -C /var/tmp -z && \
    mkdir -p /var/tmp/gromacs-2018.2/build && cd /var/tmp/gromacs-2018.2/build && cmake -DCMAKE_INSTALL_PREFIX=/usr/local/gromacs -D CMAKE_BUILD_TYPE=Release -D CUDA_TOOLKIT_ROOT_DIR=/usr/local/cuda -D GMX_BUILD_OWN_FFTW=ON -D GMX_GPU=ON -D GMX_MPI=OFF -D GMX_OPENMP=ON -D GMX_PREFER_STATIC_LIBS=ON -D MPIEXEC_PREFLAGS=--allow-run-as-root -D REGRESSIONTEST_DOWNLOAD=ON /var/tmp/gromacs-2018.2 && \
    cmake --build /var/tmp/gromacs-2018.2/build --target all -- -j$(nproc) && \
    cmake --build /var/tmp/gromacs-2018.2/build --target install -- -j$(nproc) && \
    rm -rf /var/tmp/gromacs-2018.2 /var/tmp/v2018.2.tar.gz''')

    @ubuntu
    @docker
    def test_build_directory(self):
        """build directory option"""
        g = generic_cmake(
            build_directory='/tmp/build',
            directory='spdlog-1.4.2',
            url='https://github.com/gabime/spdlog/archive/v1.4.2.tar.gz')
        self.assertEqual(str(g),
r'''# https://github.com/gabime/spdlog/archive/v1.4.2.tar.gz
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://github.com/gabime/spdlog/archive/v1.4.2.tar.gz && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/v1.4.2.tar.gz -C /var/tmp -z && \
    mkdir -p /tmp/build && cd /tmp/build && cmake -DCMAKE_INSTALL_PREFIX=/usr/local /var/tmp/spdlog-1.4.2 && \
    cmake --build /tmp/build --target all -- -j$(nproc) && \
    cmake --build /tmp/build --target install -- -j$(nproc) && \
    rm -rf /var/tmp/spdlog-1.4.2 /var/tmp/v1.4.2.tar.gz /tmp/build''')

    @ubuntu
    @docker
    def test_pre_and_post(self):
        """Preconfigure and postinstall options"""
        g = generic_cmake(
            directory='/var/tmp/spdlog-1.4.2',
            postinstall=['echo "post"'],
            preconfigure=['echo "pre"'],
            prefix='/usr/local/spdlog',
            url='https://github.com/gabime/spdlog/archive/v1.4.2.tar.gz')
        self.assertEqual(str(g),
r'''# https://github.com/gabime/spdlog/archive/v1.4.2.tar.gz
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://github.com/gabime/spdlog/archive/v1.4.2.tar.gz && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/v1.4.2.tar.gz -C /var/tmp -z && \
    cd /var/tmp/spdlog-1.4.2 && \
    echo "pre" && \
    mkdir -p /var/tmp/spdlog-1.4.2/build && cd /var/tmp/spdlog-1.4.2/build && cmake -DCMAKE_INSTALL_PREFIX=/usr/local/spdlog /var/tmp/spdlog-1.4.2 && \
    cmake --build /var/tmp/spdlog-1.4.2/build --target all -- -j$(nproc) && \
    cmake --build /var/tmp/spdlog-1.4.2/build --target install -- -j$(nproc) && \
    cd /usr/local/spdlog && \
    echo "post" && \
    rm -rf /var/tmp/spdlog-1.4.2 /var/tmp/v1.4.2.tar.gz''')

    @ubuntu
    @docker
    def test_build_environment_and_toolchain(self):
        """build environment and toolchain"""
        tc = toolchain(CC='gcc', CXX='g++', FC='gfortran')
        g = generic_cmake(
            build_environment={'FOO': 'BAR'},
            check=True,
            cmake_opts=['-D CMAKE_BUILD_TYPE=Release',
                        '-D CUDA_TOOLKIT_ROOT_DIR=/usr/local/cuda',
                        '-D GMX_BUILD_OWN_FFTW=ON',
                        '-D GMX_GPU=ON',
                        '-D GMX_MPI=OFF',
                        '-D GMX_OPENMP=ON',
                        '-D GMX_PREFER_STATIC_LIBS=ON',
                        '-D MPIEXEC_PREFLAGS=--allow-run-as-root',
                        '-D REGRESSIONTEST_DOWNLOAD=ON'],
            directory='gromacs-2018.2',
            prefix='/usr/local/gromacs',
            toolchain=tc,
            url='https://github.com/gromacs/gromacs/archive/v2018.2.tar.gz')
        self.assertEqual(str(g),
r'''# https://github.com/gromacs/gromacs/archive/v2018.2.tar.gz
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://github.com/gromacs/gromacs/archive/v2018.2.tar.gz && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/v2018.2.tar.gz -C /var/tmp -z && \
    mkdir -p /var/tmp/gromacs-2018.2/build && cd /var/tmp/gromacs-2018.2/build && FOO=BAR CC=gcc CXX=g++ FC=gfortran cmake -DCMAKE_INSTALL_PREFIX=/usr/local/gromacs -D CMAKE_BUILD_TYPE=Release -D CUDA_TOOLKIT_ROOT_DIR=/usr/local/cuda -D GMX_BUILD_OWN_FFTW=ON -D GMX_GPU=ON -D GMX_MPI=OFF -D GMX_OPENMP=ON -D GMX_PREFER_STATIC_LIBS=ON -D MPIEXEC_PREFLAGS=--allow-run-as-root -D REGRESSIONTEST_DOWNLOAD=ON /var/tmp/gromacs-2018.2 && \
    cmake --build /var/tmp/gromacs-2018.2/build --target all -- -j$(nproc) && \
    cmake --build /var/tmp/gromacs-2018.2/build --target check -- -j$(nproc) && \
    cmake --build /var/tmp/gromacs-2018.2/build --target install -- -j$(nproc) && \
    rm -rf /var/tmp/gromacs-2018.2 /var/tmp/v2018.2.tar.gz''')

    @ubuntu
    @docker
    def test_ldconfig_and_environment(self):
        """ldconfig and environment"""
        g = generic_cmake(
            devel_environment={'CPATH': '/usr/local/spdlog/include:$CPATH'},
            directory='spdlog-1.4.2',
            ldconfig=True,
            prefix='/usr/local/spdlog',
            runtime_environment={'CPATH': '/usr/local/spdlog/include:$CPATH'},
            url='https://github.com/gabime/spdlog/archive/v1.4.2.tar.gz')
        self.assertEqual(str(g),
r'''# https://github.com/gabime/spdlog/archive/v1.4.2.tar.gz
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://github.com/gabime/spdlog/archive/v1.4.2.tar.gz && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/v1.4.2.tar.gz -C /var/tmp -z && \
    mkdir -p /var/tmp/spdlog-1.4.2/build && cd /var/tmp/spdlog-1.4.2/build && cmake -DCMAKE_INSTALL_PREFIX=/usr/local/spdlog /var/tmp/spdlog-1.4.2 && \
    cmake --build /var/tmp/spdlog-1.4.2/build --target all -- -j$(nproc) && \
    cmake --build /var/tmp/spdlog-1.4.2/build --target install -- -j$(nproc) && \
    echo "/usr/local/spdlog/lib" >> /etc/ld.so.conf.d/hpccm.conf && ldconfig && \
    rm -rf /var/tmp/spdlog-1.4.2 /var/tmp/v1.4.2.tar.gz
ENV CPATH=/usr/local/spdlog/include:$CPATH''')

        r = g.runtime()
        self.assertEqual(r,
r'''# https://github.com/gabime/spdlog/archive/v1.4.2.tar.gz
COPY --from=0 /usr/local/spdlog /usr/local/spdlog
RUN echo "/usr/local/spdlog/lib" >> /etc/ld.so.conf.d/hpccm.conf && ldconfig
ENV CPATH=/usr/local/spdlog/include:$CPATH''')

    @ubuntu
    @docker
    def test_repository(self):
        """test repository option"""
        g = generic_cmake(branch='v0.8.0',
                          cmake_opts=['-D CMAKE_BUILD_TYPE=RELEASE',
                                      '-D QUDA_DIRAC_CLOVER=ON',
                                      '-D QUDA_DIRAC_DOMAIN_WALL=ON',
                                      '-D QUDA_DIRAC_STAGGERED=ON',
                                      '-D QUDA_DIRAC_TWISTED_CLOVER=ON',
                                      '-D QUDA_DIRAC_TWISTED_MASS=ON',
                                      '-D QUDA_DIRAC_WILSON=ON',
                                      '-D QUDA_FORCE_GAUGE=ON',
                                      '-D QUDA_FORCE_HISQ=ON',
                                      '-D QUDA_GPU_ARCH=sm_70',
                                      '-D QUDA_INTERFACE_MILC=ON',
                                      '-D QUDA_INTERFACE_QDP=ON',
                                      '-D QUDA_LINK_HISQ=ON',
                                      '-D QUDA_MPI=ON'],
                          prefix='/usr/local/quda',
                          recursive=True,
                          repository='https://github.com/lattice/quda.git')
        self.assertEqual(str(g),
r'''# https://github.com/lattice/quda.git
RUN mkdir -p /var/tmp && cd /var/tmp && git clone --depth=1 --branch v0.8.0 --recursive https://github.com/lattice/quda.git quda && cd - && \
    mkdir -p /var/tmp/quda/build && cd /var/tmp/quda/build && cmake -DCMAKE_INSTALL_PREFIX=/usr/local/quda -D CMAKE_BUILD_TYPE=RELEASE -D QUDA_DIRAC_CLOVER=ON -D QUDA_DIRAC_DOMAIN_WALL=ON -D QUDA_DIRAC_STAGGERED=ON -D QUDA_DIRAC_TWISTED_CLOVER=ON -D QUDA_DIRAC_TWISTED_MASS=ON -D QUDA_DIRAC_WILSON=ON -D QUDA_FORCE_GAUGE=ON -D QUDA_FORCE_HISQ=ON -D QUDA_GPU_ARCH=sm_70 -D QUDA_INTERFACE_MILC=ON -D QUDA_INTERFACE_QDP=ON -D QUDA_LINK_HISQ=ON -D QUDA_MPI=ON /var/tmp/quda && \
    cmake --build /var/tmp/quda/build --target all -- -j$(nproc) && \
    cmake --build /var/tmp/quda/build --target install -- -j$(nproc) && \
    rm -rf /var/tmp/quda''')

    @ubuntu
    @docker
    def test_runtime(self):
        """Runtime"""
        g = generic_cmake(
            cmake_opts=['-D CMAKE_BUILD_TYPE=Release',
                        '-D CUDA_TOOLKIT_ROOT_DIR=/usr/local/cuda',
                        '-D GMX_BUILD_OWN_FFTW=ON',
                        '-D GMX_GPU=ON',
                        '-D GMX_MPI=OFF',
                        '-D GMX_OPENMP=ON',
                        '-D GMX_PREFER_STATIC_LIBS=ON',
                        '-D MPIEXEC_PREFLAGS=--allow-run-as-root'],
            directory='gromacs-2018.2',
            prefix='/usr/local/gromacs',
            url='https://github.com/gromacs/gromacs/archive/v2018.2.tar.gz')
        r = g.runtime()
        self.assertEqual(r,
r'''# https://github.com/gromacs/gromacs/archive/v2018.2.tar.gz
COPY --from=0 /usr/local/gromacs /usr/local/gromacs''')

    @ubuntu
    @docker
    def test_runtime_manual(self):
        """Runtime"""
        g = generic_cmake(
            cmake_opts=['-D CMAKE_BUILD_TYPE=Release',
                        '-D CUDA_TOOLKIT_ROOT_DIR=/usr/local/cuda',
                        '-D GMX_BUILD_OWN_FFTW=ON',
                        '-D GMX_GPU=ON',
                        '-D GMX_MPI=OFF',
                        '-D GMX_OPENMP=ON',
                        '-D GMX_PREFER_STATIC_LIBS=ON',
                        '-D MPIEXEC_PREFLAGS=--allow-run-as-root'],
            directory='gromacs-2018.2',
            prefix='/usr/local/gromacs',
            runtime=['/usr/local/gromacs/bin/*', '/usr/local/gromacs/share'],
            url='https://github.com/gromacs/gromacs/archive/v2018.2.tar.gz')
        r = g.runtime()
        self.assertEqual(r,
r'''# https://github.com/gromacs/gromacs/archive/v2018.2.tar.gz
COPY --from=0 /usr/local/gromacs/bin/* /usr/local/gromacs/bin/
COPY --from=0 /usr/local/gromacs/share /usr/local/gromacs/share''')

    @ubuntu
    @docker
    def test_runtime_annotate(self):
        """Runtime"""
        g = generic_cmake(
            annotate=True,
            base_annotation='gromacs',
            cmake_opts=['-D CMAKE_BUILD_TYPE=Release',
                        '-D CUDA_TOOLKIT_ROOT_DIR=/usr/local/cuda',
                        '-D GMX_BUILD_OWN_FFTW=ON',
                        '-D GMX_GPU=ON',
                        '-D GMX_MPI=OFF',
                        '-D GMX_OPENMP=ON',
                        '-D GMX_PREFER_STATIC_LIBS=ON',
                        '-D MPIEXEC_PREFLAGS=--allow-run-as-root'],
            directory='gromacs-2018.2',
            prefix='/usr/local/gromacs',
            url='https://github.com/gromacs/gromacs/archive/v2018.2.tar.gz')
        r = g.runtime()
        self.assertEqual(r,
r'''# https://github.com/gromacs/gromacs/archive/v2018.2.tar.gz
COPY --from=0 /usr/local/gromacs /usr/local/gromacs
LABEL hpccm.gromacs.cmake='cmake -DCMAKE_INSTALL_PREFIX=/usr/local/gromacs -D CMAKE_BUILD_TYPE=Release -D CUDA_TOOLKIT_ROOT_DIR=/usr/local/cuda -D GMX_BUILD_OWN_FFTW=ON -D GMX_GPU=ON -D GMX_MPI=OFF -D GMX_OPENMP=ON -D GMX_PREFER_STATIC_LIBS=ON -D MPIEXEC_PREFLAGS=--allow-run-as-root' \
    hpccm.gromacs.url=https://github.com/gromacs/gromacs/archive/v2018.2.tar.gz''')
